@@ -107,7 +107,7 @@ ObjectPropertyConditionSet ObjectPropertyConditionSet::mergedWith(
     Vector<ObjectPropertyCondition, 16> result;
     
     if (!isEmpty())
-        result.append(m_data->begin(), m_data->size());
+        result.append(m_data->span());
     
     for (const ObjectPropertyCondition& newCondition : other) {
         bool foundMatch = false;
@@ -138,6 +138,18 @@ bool ObjectPropertyConditionSet::structuresEnsureValidity() const
     return true;
 }
 
+bool ObjectPropertyConditionSet::isStillValid() const
+{
+    if (!isValid())
+        return false;
+
+    for (const ObjectPropertyCondition& condition : *this) {
+        if (!condition.isStillValid(Concurrency::ConcurrentThread))
+            return false;
+    }
+    return true;
+}
+
 bool ObjectPropertyConditionSet::needImpurePropertyWatchpoint() const
 {
     for (const ObjectPropertyCondition& condition : *this) {
@@ -230,13 +242,23 @@ ObjectPropertyCondition generateCondition(
         break;
     }
     case PropertyCondition::Equivalence: {
-        unsigned attributes;
-        PropertyOffset offset = structure->get(vm, concurrency, uid, attributes);
-        if (offset == invalidOffset)
-            return ObjectPropertyCondition();
-        JSValue value = object->getDirect(concurrency, structure, offset);
-        if (!value)
-            return ObjectPropertyCondition();
+        JSValue value;
+        {
+            Locker<JSCellLock> cellLocker { NoLockingNecessary };
+            if (concurrency != Concurrency::MainThread) {
+                cellLocker = Locker { object->cellLock() };
+                if (object->structure() != structure)
+                    return ObjectPropertyCondition();
+                // The structure might change from now on, but we are guaranteed to have a sane view of the butterfly.
+            }
+            unsigned attributes;
+            PropertyOffset offset = structure->get(vm, concurrency, uid, attributes);
+            if (offset == invalidOffset)
+                return ObjectPropertyCondition();
+            value = object->getDirect(cellLocker, concurrency, structure, offset);
+            if (!value)
+                return ObjectPropertyCondition();
+        }
         result = ObjectPropertyCondition::equivalence(vm, owner, object, uid, value);
         break;
     }
@@ -264,7 +286,7 @@ ObjectPropertyCondition generateCondition(
 }
 
 template<typename Functor>
-ObjectPropertyConditionSet generateConditions(JSGlobalObject* globalObject, Structure* structure, JSObject* prototype, const Functor& functor)
+ObjectPropertyConditionSet generateConditions(JSGlobalObject* globalObject, Structure* structure, JSObject* prototype, UniquedStringImpl* uid, const Functor& functor)
 {
     Vector<ObjectPropertyCondition, 8> conditions;
     
@@ -286,6 +308,12 @@ ObjectPropertyConditionSet generateConditions(JSGlobalObject* globalObject, Stru
             // https://bugs.webkit.org/show_bug.cgi?id=177721
             return ObjectPropertyConditionSet::invalid();
         }
+
+        // TypedArray has an ability to stop [[Prototype]] traversing for numeric index string (e.g. "0.1").
+        // If we found it, then traverse should stop for Unset case.
+        // https://262.ecma-international.org/9.0/#_ref_2826
+        if (!prototype && isTypedArrayType(structure->typeInfo().type()) && uid && isCanonicalNumericIndexString(uid))
+            break;
         
         JSValue value = structure->prototypeForLookup(globalObject);
         
@@ -333,7 +361,7 @@ ObjectPropertyConditionSet generateConditionsForPropertyMiss(
     VM& vm, JSCell* owner, JSGlobalObject* globalObject, Structure* headStructure, UniquedStringImpl* uid)
 {
     return generateConditions(
-        globalObject, headStructure, nullptr,
+        globalObject, headStructure, nullptr, uid,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             ObjectPropertyCondition result =
                 generateCondition(vm, owner, object, structure, uid, PropertyCondition::Absence, Concurrency::MainThread);
@@ -348,7 +376,7 @@ ObjectPropertyConditionSet generateConditionsForPropertySetterMiss(
     VM& vm, JSCell* owner, JSGlobalObject* globalObject, Structure* headStructure, UniquedStringImpl* uid)
 {
     return generateConditions(
-        globalObject, headStructure, nullptr,
+        globalObject, headStructure, nullptr, uid,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             ObjectPropertyCondition result =
                 generateCondition(vm, owner, object, structure, uid, PropertyCondition::AbsenceOfSetEffect, Concurrency::MainThread);
@@ -362,7 +390,7 @@ ObjectPropertyConditionSet generateConditionsForPropertySetterMiss(
 ObjectPropertyConditionSet generateConditionsForIndexedMiss(VM& vm, JSCell* owner, JSGlobalObject* globalObject, Structure* headStructure)
 {
     return generateConditions(
-        globalObject, headStructure, nullptr,
+        globalObject, headStructure, nullptr, nullptr,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             ObjectPropertyCondition result =
                 generateCondition(vm, owner, object, structure, nullptr, PropertyCondition::AbsenceOfIndexedProperties, Concurrency::MainThread);
@@ -378,7 +406,7 @@ ObjectPropertyConditionSet generateConditionsForPrototypePropertyHit(
     UniquedStringImpl* uid)
 {
     return generateConditions(
-        globalObject, headStructure, prototype,
+        globalObject, headStructure, prototype, uid,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             PropertyCondition::Kind kind =
                 object == prototype ? PropertyCondition::Presence : PropertyCondition::Absence;
@@ -396,7 +424,7 @@ ObjectPropertyConditionSet generateConditionsForPrototypePropertyHitCustom(
     UniquedStringImpl* uid, unsigned attributes)
 {
     return generateConditions(
-        globalObject, headStructure, prototype,
+        globalObject, headStructure, prototype, uid,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             auto kind = PropertyCondition::Absence;
             if (object == prototype) {
@@ -446,7 +474,7 @@ ObjectPropertyConditionSet generateConditionsForInstanceOf(
     if (ObjectPropertyConditionSetInternal::verbose)
         dataLog("Searching for prototype ", JSValue(prototype), " starting with structure ", RawPointer(headStructure), " with shouldHit = ", shouldHit, "\n");
     ObjectPropertyConditionSet result = generateConditions(
-        globalObject, headStructure, shouldHit ? prototype : nullptr,
+        globalObject, headStructure, shouldHit ? prototype : nullptr, nullptr,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             if (ObjectPropertyConditionSetInternal::verbose)
                 dataLog("Encountered object: ", RawPointer(object), "\n");
@@ -474,7 +502,7 @@ ObjectPropertyConditionSet generateConditionsForInstanceOf(
 ObjectPropertyConditionSet generateConditionsForPrototypeEquivalenceConcurrently(
     VM& vm, JSGlobalObject* globalObject, Structure* headStructure, JSObject* prototype, UniquedStringImpl* uid)
 {
-    return generateConditions(globalObject, headStructure, prototype,
+    return generateConditions(globalObject, headStructure, prototype, uid,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             PropertyCondition::Kind kind =
                 object == prototype ? PropertyCondition::Equivalence : PropertyCondition::Absence;
@@ -486,11 +514,25 @@ ObjectPropertyConditionSet generateConditionsForPrototypeEquivalenceConcurrently
         });
 }
 
-ObjectPropertyConditionSet generateConditionsForPropertyMissConcurrently(
-    VM& vm, JSGlobalObject* globalObject, Structure* headStructure, UniquedStringImpl* uid)
+ObjectPropertyConditionSet generateConditionsForPrototypePropertyHitConcurrently(VM& vm, JSGlobalObject* globalObject, Structure* headStructure, JSObject* prototype, UniquedStringImpl* uid)
 {
     return generateConditions(
-        globalObject, headStructure, nullptr,
+        globalObject, headStructure, prototype, uid,
+        [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
+            PropertyCondition::Kind kind = object == prototype ? PropertyCondition::Presence : PropertyCondition::Absence;
+            ObjectPropertyCondition result =
+                generateCondition(vm, nullptr, object, structure, uid, kind, Concurrency::ConcurrentThread);
+            if (!result)
+                return false;
+            conditions.append(result);
+            return true;
+        });
+}
+
+ObjectPropertyConditionSet generateConditionsForPropertyMissConcurrently(VM& vm, JSGlobalObject* globalObject, Structure* headStructure, UniquedStringImpl* uid)
+{
+    return generateConditions(
+        globalObject, headStructure, nullptr, uid,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             ObjectPropertyCondition result = generateCondition(vm, nullptr, object, structure, uid, PropertyCondition::Absence, Concurrency::ConcurrentThread);
             if (!result)
@@ -504,7 +546,7 @@ ObjectPropertyConditionSet generateConditionsForPropertySetterMissConcurrently(
     VM& vm, JSGlobalObject* globalObject, Structure* headStructure, UniquedStringImpl* uid)
 {
     return generateConditions(
-        globalObject, headStructure, nullptr,
+        globalObject, headStructure, nullptr, uid,
         [&](auto& conditions, JSObject* object, Structure* structure) -> bool {
             ObjectPropertyCondition result =
                 generateCondition(vm, nullptr, object, structure, uid, PropertyCondition::AbsenceOfSetEffect, Concurrency::ConcurrentThread);
@@ -522,7 +564,7 @@ ObjectPropertyCondition generateConditionForSelfEquivalence(
 }
 
 // Current might be null. Structure can't be null.
-static std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, JSCell* current, Structure* structure, JSObject* target)
+static std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, JSCell* current, Structure* structure, UniquedStringImpl* propertyName, JSObject* target)
 {
     ASSERT(structure);
     VM& vm = globalObject->vm();
@@ -537,8 +579,19 @@ static std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGloba
                 return std::nullopt;
 
             ASSERT(structure->isObject());
-            if (structure->hasBeenFlattenedBefore())
+            if (structure->hasBeenFlattenedBefore()) {
+                if (structure->isUncacheableDictionary())
+                    return std::nullopt;
+                if (!structure->propertyAccessesAreCacheable())
+                    return std::nullopt;
+                if (structure->isProxy())
+                    return std::nullopt;
+                if (current == target) {
+                    found = true;
+                    break;
+                }
                 return std::nullopt;
+            }
 
             structure->flattenDictionaryStructure(vm, asObject(current));
             flattenedDictionary = true;
@@ -555,6 +608,14 @@ static std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGloba
             break;
         }
 
+        // TypedArray has an ability to stop [[Prototype]] traversing for numeric index string (e.g. "0.1").
+        // If we found it, then traverse should stop for Unset case.
+        // https://262.ecma-international.org/9.0/#_ref_2826
+        if (!target && propertyName && isTypedArrayType(structure->typeInfo().type()) && isCanonicalNumericIndexString(propertyName)) {
+            found = true;
+            break;
+        }
+
         // We only have poly proto if we need to access our prototype via
         // the poly proto protocol. If the slot base is the only poly proto
         // thing in the chain, and we have a cache hit on it, then we're not
@@ -584,20 +645,20 @@ static std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGloba
     return result;
 }
 
-std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, JSCell* base, JSObject* target)
+std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, JSCell* base, UniquedStringImpl* propertyName, JSObject* target)
 {
-    return prepareChainForCaching(globalObject, base, base->structure(), target);
+    return prepareChainForCaching(globalObject, base, base->structure(), propertyName, target);
 }
 
-std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, JSCell* base, const PropertySlot& slot)
+std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, JSCell* base, UniquedStringImpl* propertyName, const PropertySlot& slot)
 {
     JSObject* target = slot.isUnset() ? nullptr : slot.slotBase();
-    return prepareChainForCaching(globalObject, base, target);
+    return prepareChainForCaching(globalObject, base, propertyName, target);
 }
 
-std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, Structure* baseStructure, JSObject* target)
+std::optional<PrototypeChainCachingStatus> prepareChainForCaching(JSGlobalObject* globalObject, Structure* baseStructure, UniquedStringImpl* propertyName, JSObject* target)
 {
-    return prepareChainForCaching(globalObject, nullptr, baseStructure, target);
+    return prepareChainForCaching(globalObject, nullptr, baseStructure, propertyName, target);
 }
 
 } // namespace JSC
