@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,11 +39,14 @@
 #include "LLIntData.h"
 #include "LLIntSlowPaths.h"
 #include "JSCInlines.h"
+#include "SuperSampler.h"
 #include <wtf/Assertions.h>
 #include <wtf/MathExtras.h>
 
 using namespace JSC::LLInt;
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 // LLInt C Loop opcodes
 // ====================
 // In the implementation of the C loop, the LLint trampoline glue functions
@@ -54,7 +57,7 @@ using namespace JSC::LLInt;
 // includes.
 //
 // In addition, some JIT trampoline functions which are needed by LLInt
-// (e.g. getHostCallReturnValue, ctiOpThrowNotCaught) are also added as
+// (e.g. ctiOpThrowNotCaught) are also added as
 // bytecodes, and the CLoop will provide bytecode handlers for them.
 //
 // In the CLoop, we can only dispatch indirectly to these bytecodes
@@ -87,6 +90,8 @@ using namespace JSC::LLInt;
 // Define the opcode dispatch mechanism when using the C loop:
 //
 
+using WebConfig::g_config;
+
 // These are for building a C Loop interpreter:
 #define OFFLINE_ASM_BEGIN
 #define OFFLINE_ASM_END
@@ -125,6 +130,8 @@ using namespace JSC::LLInt;
 
 namespace JSC {
 
+class CallLinkInfo;
+
 //============================================================================
 // CLoopRegister is the storage for an emulated CPU register.
 // It defines the policy of how ints smaller than intptr_t are packed into the
@@ -139,32 +146,33 @@ public:
     ALWAYS_INLINE int8_t i8() const { return m_value; }
     ALWAYS_INLINE uint8_t u8() const { return m_value; }
 
-    ALWAYS_INLINE intptr_t* ip() const { return bitwise_cast<intptr_t*>(m_value); }
-    ALWAYS_INLINE int8_t* i8p() const { return bitwise_cast<int8_t*>(m_value); }
-    ALWAYS_INLINE void* vp() const { return bitwise_cast<void*>(m_value); }
-    ALWAYS_INLINE const void* cvp() const { return bitwise_cast<const void*>(m_value); }
-    ALWAYS_INLINE CallFrame* callFrame() const { return bitwise_cast<CallFrame*>(m_value); }
-    ALWAYS_INLINE const void* instruction() const { return bitwise_cast<const void*>(m_value); }
-    ALWAYS_INLINE VM* vm() const { return bitwise_cast<VM*>(m_value); }
-    ALWAYS_INLINE JSCell* cell() const { return bitwise_cast<JSCell*>(m_value); }
-    ALWAYS_INLINE ProtoCallFrame* protoCallFrame() const { return bitwise_cast<ProtoCallFrame*>(m_value); }
-    ALWAYS_INLINE NativeFunction nativeFunc() const { return bitwise_cast<NativeFunction>(m_value); }
+    ALWAYS_INLINE intptr_t* ip() const { return std::bit_cast<intptr_t*>(m_value); }
+    ALWAYS_INLINE int8_t* i8p() const { return std::bit_cast<int8_t*>(m_value); }
+    ALWAYS_INLINE void* vp() const { return std::bit_cast<void*>(m_value); }
+    ALWAYS_INLINE const void* cvp() const { return std::bit_cast<const void*>(m_value); }
+    ALWAYS_INLINE CallFrame* callFrame() const { return std::bit_cast<CallFrame*>(m_value); }
+    ALWAYS_INLINE const void* instruction() const { return std::bit_cast<const void*>(m_value); }
+    ALWAYS_INLINE VM* vm() const { return std::bit_cast<VM*>(m_value); }
+    ALWAYS_INLINE JSCell* cell() const { return std::bit_cast<JSCell*>(m_value); }
+    ALWAYS_INLINE ProtoCallFrame* protoCallFrame() const { return std::bit_cast<ProtoCallFrame*>(m_value); }
+    ALWAYS_INLINE NativeFunction nativeFunc() const { return std::bit_cast<NativeFunction>(m_value); }
 #if USE(JSVALUE64)
     ALWAYS_INLINE int64_t i64() const { return m_value; }
     ALWAYS_INLINE uint64_t u64() const { return m_value; }
-    ALWAYS_INLINE EncodedJSValue encodedJSValue() const { return bitwise_cast<EncodedJSValue>(m_value); }
+    ALWAYS_INLINE EncodedJSValue encodedJSValue() const { return std::bit_cast<EncodedJSValue>(m_value); }
 #endif
-    ALWAYS_INLINE Opcode opcode() const { return bitwise_cast<Opcode>(m_value); }
+    ALWAYS_INLINE Opcode opcode() const { return std::bit_cast<Opcode>(m_value); }
 
-    operator CallFrame*() { return bitwise_cast<CallFrame*>(m_value); }
-    operator const Instruction*() { return bitwise_cast<const Instruction*>(m_value); }
-    operator JSCell*() { return bitwise_cast<JSCell*>(m_value); }
-    operator ProtoCallFrame*() { return bitwise_cast<ProtoCallFrame*>(m_value); }
-    operator Register*() { return bitwise_cast<Register*>(m_value); }
-    operator VM*() { return bitwise_cast<VM*>(m_value); }
+    operator CallFrame*() { return std::bit_cast<CallFrame*>(m_value); }
+    operator const JSInstruction*() { return std::bit_cast<const JSInstruction*>(m_value); }
+    operator JSCell*() { return std::bit_cast<JSCell*>(m_value); }
+    operator ProtoCallFrame*() { return std::bit_cast<ProtoCallFrame*>(m_value); }
+    operator Register*() { return std::bit_cast<Register*>(m_value); }
+    operator VM*() { return std::bit_cast<VM*>(m_value); }
+    operator CallLinkInfo*() { return std::bit_cast<CallLinkInfo*>(m_value); }
 
     template<typename T, typename = std::enable_if_t<sizeof(T) == sizeof(uintptr_t)>>
-    ALWAYS_INLINE void operator=(T value) { m_value = bitwise_cast<uintptr_t>(value); }
+    ALWAYS_INLINE void operator=(T value) { m_value = std::bit_cast<uintptr_t>(value); }
 #if USE(JSVALUE64)
     ALWAYS_INLINE void operator=(int32_t value) { m_value = static_cast<intptr_t>(value); }
     ALWAYS_INLINE void operator=(uint32_t value) { m_value = static_cast<uintptr_t>(value); }
@@ -176,8 +184,8 @@ public:
     ALWAYS_INLINE void operator=(bool value) { m_value = static_cast<uintptr_t>(value); }
 
 #if USE(JSVALUE64)
-    ALWAYS_INLINE double bitsAsDouble() const { return bitwise_cast<double>(m_value); }
-    ALWAYS_INLINE int64_t bitsAsInt64() const { return bitwise_cast<int64_t>(m_value); }
+    ALWAYS_INLINE double bitsAsDouble() const { return std::bit_cast<double>(m_value); }
+    ALWAYS_INLINE int64_t bitsAsInt64() const { return std::bit_cast<int64_t>(m_value); }
 #endif
 
 private:
@@ -187,15 +195,15 @@ private:
 class CLoopDoubleRegister {
 public:
     template<typename T>
-    explicit operator T() const { return bitwise_cast<T>(m_value); }
+    explicit operator T() const { return std::bit_cast<T>(m_value); }
 
     ALWAYS_INLINE double d() const { return m_value; }
-    ALWAYS_INLINE int64_t bitsAsInt64() const { return bitwise_cast<int64_t>(m_value); }
+    ALWAYS_INLINE int64_t bitsAsInt64() const { return std::bit_cast<int64_t>(m_value); }
 
     ALWAYS_INLINE void operator=(double value) { m_value = value; }
 
     template<typename T, typename = std::enable_if_t<sizeof(T) == sizeof(uintptr_t) && std::is_integral<T>::value>>
-    ALWAYS_INLINE void operator=(T value) { m_value = bitwise_cast<double>(value); }
+    ALWAYS_INLINE void operator=(T value) { m_value = std::bit_cast<double>(value); }
 
 private:
     double m_value;
@@ -211,18 +219,18 @@ namespace LLInt {
 static double ints2Double(uint32_t lo, uint32_t hi)
 {
     uint64_t value = (static_cast<uint64_t>(hi) << 32) | lo;
-    return bitwise_cast<double>(value);
+    return std::bit_cast<double>(value);
 }
 
 static void double2Ints(double val, CLoopRegister& lo, CLoopRegister& hi)
 {
-    uint64_t value = bitwise_cast<uint64_t>(val);
+    uint64_t value = std::bit_cast<uint64_t>(val);
     hi = static_cast<uint32_t>(value >> 32);
     lo = static_cast<uint32_t>(value);
 }
 #endif // USE(JSVALUE32_64)
 
-static void decodeResult(SlowPathReturnType result, CLoopRegister& t0, CLoopRegister& t1)
+static void decodeResult(UGPRPair result, CLoopRegister& t0, CLoopRegister& t1)
 {
     const void* t0Result;
     const void* t1Result;
@@ -239,26 +247,30 @@ static void decodeResult(SlowPathReturnType result, CLoopRegister& t0, CLoopRegi
 
 JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm, ProtoCallFrame* protoCallFrame, bool isInitializationPass)
 {
-#define CAST bitwise_cast
+    // WARNING: it is critical to avoid 'std::bit_cast' in the CAST macro that follows and in
+    // the OPCODE_ENTRY() and LLINT_OPCODE_ENTRY() macros below. Using it dramatically increases the
+    // frame size of this function in debug builds, potentially causing stack overflows in tests.
+    // See https://bugs.webkit.org/show_bug.cgi?id=295796 for details.
+#define CAST reinterpret_cast
 
     // One-time initialization of our address tables. We have to put this code
     // here because our labels are only in scope inside this function. The
     // caller (or one of its ancestors) is responsible for ensuring that this
     // is only called once during the initialization of the VM before threads
     // are at play.
-    if (UNLIKELY(isInitializationPass)) {
+    if (isInitializationPass) [[unlikely]] {
         Opcode* opcodeMap = LLInt::opcodeMap();
         Opcode* opcodeMapWide16 = LLInt::opcodeMapWide16();
         Opcode* opcodeMapWide32 = LLInt::opcodeMapWide32();
 
 #if ENABLE(COMPUTED_GOTO_OPCODES)
         #define OPCODE_ENTRY(__opcode, length) \
-            opcodeMap[__opcode] = bitwise_cast<void*>(&&__opcode); \
-            opcodeMapWide16[__opcode] = bitwise_cast<void*>(&&__opcode##_wide16); \
-            opcodeMapWide32[__opcode] = bitwise_cast<void*>(&&__opcode##_wide32);
+            opcodeMap[__opcode] = static_cast<void*>(&&__opcode); \
+            opcodeMapWide16[__opcode] = static_cast<void*>(&&__opcode##_wide16); \
+            opcodeMapWide32[__opcode] = static_cast<void*>(&&__opcode##_wide32);
 
         #define LLINT_OPCODE_ENTRY(__opcode, length) \
-            opcodeMap[__opcode] = bitwise_cast<void*>(&&__opcode);
+            opcodeMap[__opcode] = static_cast<void*>(&&__opcode);
 #else
         // FIXME: this mapping is unnecessarily expensive in the absence of COMPUTED_GOTO
         //   narrow opcodes don't need any mapping and wide opcodes just need to add numOpcodeIDs
@@ -271,8 +283,10 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
             opcodeMap[__opcode] = __opcode;
 #endif
         FOR_EACH_BYTECODE_ID(OPCODE_ENTRY)
+        FOR_EACH_BYTECODE_HELPER_ID(OPCODE_ENTRY)
         FOR_EACH_CLOOP_BYTECODE_HELPER_ID(LLINT_OPCODE_ENTRY)
         FOR_EACH_LLINT_NATIVE_HELPER(LLINT_OPCODE_ENTRY)
+        FOR_EACH_CLOOP_RETURN_HELPER_ID(LLINT_OPCODE_ENTRY)
         #undef OPCODE_ENTRY
         #undef LLINT_OPCODE_ENTRY
 
@@ -280,7 +294,7 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
         // initialized the opcodeMap above. This is because getCodePtr()
         // can depend on the opcodeMap.
         uint8_t* exceptionInstructions = reinterpret_cast<uint8_t*>(LLInt::exceptionInstructions());
-        for (unsigned i = 0; i < maxOpcodeLength + 1; ++i)
+        for (unsigned i = 0; i < maxBytecodeStructLength + 1; ++i)
             exceptionInstructions[i] = llint_throw_from_slow_path_trampoline;
 
         return JSValue();
@@ -313,7 +327,7 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
     // 2. 32 bit result values will be in the low 32-bit of t0.
     // 3. 64 bit result values will be in t0.
 
-    CLoopRegister t0, t1, t2, t3, t5, sp, cfr, lr, pc;
+    CLoopRegister t0, t1, t2, t3, t5, t6, t7, sp, cfr, lr, pc;
 #if USE(JSVALUE64)
     CLoopRegister numberTag, notCellMask;
 #endif
@@ -321,6 +335,14 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
     CLoopRegister metadataTable;
     CLoopDoubleRegister d0, d1;
 
+    UNUSED_VARIABLE(t0);
+    UNUSED_VARIABLE(t1);
+    UNUSED_VARIABLE(t2);
+    UNUSED_VARIABLE(t3);
+    UNUSED_VARIABLE(t5);
+    UNUSED_VARIABLE(t6);
+    UNUSED_VARIABLE(t7);
+
     struct StackPointerScope {
         StackPointerScope(CLoopStack& stack)
             : m_stack(stack)
@@ -337,7 +359,7 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
         void* m_originalStackPointer;
     };
 
-    CLoopStack& cloopStack = vm->interpreter->cloopStack();
+    CLoopStack& cloopStack = vm->interpreter.cloopStack();
     StackPointerScope stackPointerScope(cloopStack);
 
     lr = getOpcode(llint_return_to_host);
@@ -436,23 +458,6 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 #endif
         }
 
-        // In the ASM llint, getHostCallReturnValue() is a piece of glue
-        // function provided by the JIT (see jit/JITOperations.cpp).
-        // We simulate it here with a pseduo-opcode handler.
-        OFFLINE_ASM_GLUE_LABEL(getHostCallReturnValue)
-        {
-            // The part in getHostCallReturnValueWithExecState():
-            JSValue result = vm->hostCallReturnValue;
-#if USE(JSVALUE32_64)
-            t1 = result.tag();
-            t0 = result.payload();
-#else
-            t0 = JSValue::encode(result);
-#endif
-            opcode = lr.opcode();
-            DISPATCH_OPCODE();
-        }
-
 #if !ENABLE(COMPUTED_GOTO_OPCODES)
     default:
         ASSERT(false);
@@ -482,15 +487,46 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 
 } // namespace JSC
 
-#elif !COMPILER(MSVC)
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
+
+#else // !ENABLE(C_LOOP)
 
 //============================================================================
 // Define the opcode dispatch mechanism when using an ASM loop:
 //
 
+// We need an OFFLINE_ASM_BEGIN_SPACER because we'll be declaring every OFFLINE_ASM_GLOBAL_LABEL
+// as an alt entry. However, Clang will error out if the first global label is also an alt entry.
+// To work around this, we'll make OFFLINE_ASM_BEGIN emit an unused global label (which will now
+// be the first) that is not an alt entry, and insert a spacer instruction between it and the
+// actual first global label emitted by the offlineasm. Clang also requires that these 2 labels
+// not point to the same spot in memory; hence, the need for the spacer.
+//
+// For the spacer instruction, we'll choose a breakpoint instruction. However, we can
+// also just emit an unused piece of data. A breakpoint instruction is preferable.
+
+#if CPU(ARM_THUMB2)
+#define OFFLINE_ASM_BEGIN_SPACER "bkpt #0\n"
+#elif CPU(ARM64)
+#define OFFLINE_ASM_BEGIN_SPACER "brk #" STRINGIZE_VALUE_OF(WTF_FATAL_CRASH_CODE) "\n"
+#elif CPU(X86_64)
+#define OFFLINE_ASM_BEGIN_SPACER "int3\n"
+#else
+#define OFFLINE_ASM_BEGIN_SPACER ".int 0xbadbeef0\n"
+#endif
+
 // These are for building an interpreter from generated assembly code:
-#define OFFLINE_ASM_BEGIN   asm (
-#define OFFLINE_ASM_END     );
+// the jsc_llint_begin and jsc_llint_end labels help lldb_webkit.py find the
+// start and end of the llint instruction range quickly.
+
+#define OFFLINE_ASM_BEGIN   asm ( \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(jsc_llint_begin, OFFLINE_ASM_NO_ALT_ENTRY_DIRECTIVE, OFFLINE_ASM_ALIGN4B, HIDE_SYMBOL) \
+    OFFLINE_ASM_BEGIN_SPACER
+
+#define OFFLINE_ASM_END \
+    OFFLINE_ASM_BEGIN_SPACER \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(jsc_llint_end, OFFLINE_ASM_NO_ALT_ENTRY_DIRECTIVE, OFFLINE_ASM_ALIGN4B, HIDE_SYMBOL) \
+);
 
 #if ENABLE(LLINT_EMBEDDED_OPCODE_ID)
 #define EMBED_OPCODE_ID_IF_NEEDED(__opcode) ".int " __opcode##_value_string "\n"
@@ -503,33 +539,89 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
     OFFLINE_ASM_OPCODE_DEBUG_LABEL(llint_##__opcode) \
     OFFLINE_ASM_LOCAL_LABEL(llint_##__opcode)
 
-#define OFFLINE_ASM_GLUE_LABEL(__opcode)   OFFLINE_ASM_LOCAL_LABEL(__opcode)
+#define OFFLINE_ASM_GLUE_LABEL(__opcode) \
+    OFFLINE_ASM_OPCODE_DEBUG_LABEL(__opcode) \
+    OFFLINE_ASM_LOCAL_LABEL(__opcode)
+
+#define OFFLINE_ASM_NO_ALT_ENTRY_DIRECTIVE(label)
+
+#if COMPILER(CLANG) && OS(DARWIN) && ENABLE(OFFLINE_ASM_ALT_ENTRY)
+#define OFFLINE_ASM_ALT_ENTRY_DIRECTIVE(label) \
+    ".alt_entry " SYMBOL_STRING(label) "\n"
+#else
+#define OFFLINE_ASM_ALT_ENTRY_DIRECTIVE(label)
+#endif
+
+#if OS(DARWIN)
+#define OFFLINE_ASM_TEXT_SECTION ".section __TEXT,__jsc_int,regular,pure_instructions\n"
+#else
+#define OFFLINE_ASM_TEXT_SECTION ".text\n"
+#endif
 
 #if CPU(ARM_THUMB2)
-#define OFFLINE_ASM_GLOBAL_LABEL(label)          \
-    ".text\n"                                    \
-    ".align 4\n"                                 \
+#define OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, ALT_ENTRY, ALIGNMENT, VISIBILITY) \
+    OFFLINE_ASM_TEXT_SECTION                     \
+    ALIGNMENT                                    \
+    ALT_ENTRY(label)                             \
     ".globl " SYMBOL_STRING(label) "\n"          \
-    HIDE_SYMBOL(label) "\n"                      \
+    VISIBILITY(label) "\n"                       \
     ".thumb\n"                                   \
     ".thumb_func " THUMB_FUNC_PARAM(label) "\n"  \
     SYMBOL_STRING(label) ":\n"
-#elif CPU(ARM64)
-#define OFFLINE_ASM_GLOBAL_LABEL(label)         \
-    ".text\n"                                   \
-    ".align 4\n"                                \
+#elif CPU(RISCV64)
+#define OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, ALT_ENTRY, ALIGNMENT, VISIBILITY) \
+    OFFLINE_ASM_TEXT_SECTION                    \
+    ALIGNMENT                                   \
+    ALT_ENTRY(label)                            \
     ".globl " SYMBOL_STRING(label) "\n"         \
-    HIDE_SYMBOL(label) "\n"                     \
+    ".attribute arch, \"rv64gc\"" "\n"          \
+    VISIBILITY(label) "\n"                      \
     SYMBOL_STRING(label) ":\n"
 #else
-#define OFFLINE_ASM_GLOBAL_LABEL(label)         \
-    ".text\n"                                   \
+#define OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, ALT_ENTRY, ALIGNMENT, VISIBILITY) \
+    OFFLINE_ASM_TEXT_SECTION                    \
+    ALIGNMENT                                   \
+    ALT_ENTRY(label)                            \
     ".globl " SYMBOL_STRING(label) "\n"         \
-    HIDE_SYMBOL(label) "\n"                     \
+    VISIBILITY(label) "\n"                      \
     SYMBOL_STRING(label) ":\n"
 #endif
 
-#define OFFLINE_ASM_LOCAL_LABEL(label)   LOCAL_LABEL_STRING(label) ":\n"
+#define OFFLINE_ASM_ALIGN4B ".balign 4\n"
+#define OFFLINE_ASM_NOALIGN ""
+
+#if CPU(ARM64) || CPU(ARM64E)
+#define OFFLINE_ASM_ALIGN_TRAP(align) OFFLINE_ASM_BEGIN_SPACER "\n .balignl " #align ", 0xd4388e20\n" // pad with brk instructions
+#elif CPU(X86_64)
+#define OFFLINE_ASM_ALIGN_TRAP(align) OFFLINE_ASM_BEGIN_SPACER "\n .balign " #align ", 0xcc\n" // pad with int 3 instructions
+#elif CPU(ARM)
+#define OFFLINE_ASM_ALIGN_TRAP(align) OFFLINE_ASM_BEGIN_SPACER "\n .balignw " #align ", 0xde00\n" // pad with udf instructions
+#elif CPU(RISCV64)
+#define OFFLINE_ASM_ALIGN_TRAP(align) OFFLINE_ASM_BEGIN_SPACER "\n .balignw " #align ", 0x9002\n" // pad with c.ebreak instructions
+#endif
+
+#define OFFLINE_ASM_EXPORT_SYMBOL(symbol)
+
+#define OFFLINE_ASM_GLOBAL_LABEL(label) \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, OFFLINE_ASM_ALT_ENTRY_DIRECTIVE, OFFLINE_ASM_ALIGN4B, HIDE_SYMBOL)
+#define OFFLINE_ASM_UNALIGNED_GLOBAL_LABEL(label) \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, OFFLINE_ASM_ALT_ENTRY_DIRECTIVE, OFFLINE_ASM_NOALIGN, HIDE_SYMBOL)
+#define OFFLINE_ASM_ALIGNED_GLOBAL_LABEL(label, align) \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, OFFLINE_ASM_ALT_ENTRY_DIRECTIVE, OFFLINE_ASM_ALIGN_TRAP(align), HIDE_SYMBOL)
+#define OFFLINE_ASM_GLOBAL_EXPORT_LABEL(label) \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, OFFLINE_ASM_ALT_ENTRY_DIRECTIVE, OFFLINE_ASM_ALIGN4B, OFFLINE_ASM_EXPORT_SYMBOL)
+#define OFFLINE_ASM_UNALIGNED_GLOBAL_EXPORT_LABEL(label) \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, OFFLINE_ASM_ALT_ENTRY_DIRECTIVE, OFFLINE_ASM_NOALIGN, OFFLINE_ASM_EXPORT_SYMBOL)
+
+#if COMPILER(CLANG) && ENABLE(OFFLINE_ASM_ALT_ENTRY)
+#define OFFLINE_ASM_ALT_GLOBAL_LABEL(label) OFFLINE_ASM_GLOBAL_LABEL(label)
+#else
+#define OFFLINE_ASM_ALT_GLOBAL_LABEL(label)
+#endif
+
+#define OFFLINE_ASM_LOCAL_LABEL(label) \
+    LOCAL_LABEL_STRING(label) ":\n" \
+    OFFLINE_ASM_ALT_GLOBAL_LABEL(label)
 
 #if OS(LINUX)
 #define OFFLINE_ASM_OPCODE_DEBUG_LABEL(label)  #label ":\n"
@@ -537,8 +629,71 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 #define OFFLINE_ASM_OPCODE_DEBUG_LABEL(label)
 #endif
 
+#include "WasmCallee.h"
+
+// This works around a bug in GDB where, if the compilation unit
+// doesn't have any address range information, its line table won't
+// even be consulted. Emit {before,after}_llint_asm so that the code
+// emitted in the top level inline asm statement is within functions
+// visible to the compiler. This way, GDB can resolve a PC in the
+// llint asm code to this compilation unit and the successfully look
+// up the line number information.
+DEBUGGER_ANNOTATION_MARKER(before_llint_asm)
+
+
+// We do not set them on Darwin since Mach-O does not support nested cfi_startproc & global symbols.
+// https://github.com/llvm/llvm-project/issues/72802
+// Similarly, GCC complains about implicit endproc instructions.
+//
+// This may seem strange; We duplicate these table entries because
+// different lldb versions seem to sometimes have off-by-one errors otherwise.
+// See GdbJIT.cpp for a detailed explanation of how these DWARF directives work.
+#if !OS(DARWIN) && COMPILER(CLANG)
+#if CPU(ARM64)
+asm (
+    ".cfi_startproc\n"
+    ".cfi_def_cfa fp, 16\n"
+    ".cfi_offset lr, -8\n"
+    ".cfi_offset fp, -16\n"
+    OFFLINE_ASM_BEGIN_SPACER
+    ".cfi_def_cfa fp, 0\n"
+    ".cfi_offset lr, 0\n"
+    ".cfi_offset fp, 0\n"
+    OFFLINE_ASM_BEGIN_SPACER
+    ".cfi_def_cfa fp, 16\n"
+    ".cfi_offset lr, -8\n"
+    ".cfi_offset fp, -16\n"
+    OFFLINE_ASM_BEGIN_SPACER
+);
+#elif CPU(ARM_THUMB2)
+asm (
+    ".cfi_startproc\n"
+    OFFLINE_ASM_BEGIN_SPACER
+    ".cfi_def_cfa r7, 8\n"
+    ".cfi_offset lr, -4\n"
+    ".cfi_offset fp, -8\n"
+    OFFLINE_ASM_BEGIN_SPACER
+    ".cfi_def_cfa r7, 8\n"
+    ".cfi_offset lr, -4\n"
+    ".cfi_offset fp, -8\n"
+    OFFLINE_ASM_BEGIN_SPACER
+);
+#endif
+#endif
+
 // This is a file generated by offlineasm, which contains all of the assembly code
 // for the interpreter, as compiled from LowLevelInterpreter.asm.
 #include "LLIntAssembly.h"
 
+// See GdbJIT.cpp for a detailed explanation.
+#if !OS(DARWIN) && COMPILER(CLANG)
+#if CPU(ARM64) || CPU(ARM_THUMB2)
+asm (
+    ".cfi_endproc\n"
+);
+#endif
+#endif
+
+DEBUGGER_ANNOTATION_MARKER(after_llint_asm)
+
 #endif // ENABLE(C_LOOP)
