@@ -30,16 +30,10 @@
 
 #include "AuxiliaryBarrierInlines.h"
 #include "JSCInlines.h"
-#include "WebAssemblyRuntimeErrorPrototype.lut.h"
 
 namespace JSC {
 
-const ClassInfo WebAssemblyRuntimeErrorPrototype::s_info = { "WebAssembly.RuntimeError"_s, &Base::s_info, &prototypeTableWebAssemblyRuntimeError, nullptr, CREATE_METHOD_TABLE(WebAssemblyRuntimeErrorPrototype) };
-
-/* Source for WebAssemblyRuntimeErrorPrototype.lut.h
- @begin prototypeTableWebAssemblyRuntimeError
- @end
- */
+const ClassInfo WebAssemblyRuntimeErrorPrototype::s_info = { "WebAssembly.RuntimeError"_s, &Base::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(WebAssemblyRuntimeErrorPrototype) };
 
 WebAssemblyRuntimeErrorPrototype* WebAssemblyRuntimeErrorPrototype::create(VM& vm, JSGlobalObject*, Structure* structure)
 {
