@@ -43,12 +43,14 @@ BlockType GetBlockType(TQualifier qualifier)
     switch (qualifier)
     {
         case EvqUniform:
-            return BlockType::BLOCK_UNIFORM;
+            return BlockType::kBlockUniform;
         case EvqBuffer:
-            return BlockType::BLOCK_BUFFER;
+            return BlockType::kBlockBuffer;
+        case EvqPixelLocalEXT:
+            return BlockType::kPixelLocalExt;
         default:
             UNREACHABLE();
-            return BlockType::BLOCK_UNIFORM;
+            return BlockType::kBlockUniform;
     }
 }
 
@@ -147,6 +149,7 @@ class CollectVariablesTraverser : public TIntermTraverser
                             bool staticUse,
                             bool isShaderIOBlock,
                             bool isPatch,
+                            SymbolType symbolType,
                             ShaderVariable *variableOut) const;
     void setCommonVariableProperties(const TType &type,
                                      const TVariable &variable,
@@ -208,6 +211,7 @@ class CollectVariablesTraverser : public TIntermTraverser
     bool mHelperInvocationAdded;
     bool mFragCoordAdded;
     bool mLastFragDataAdded;
+    bool mLastFragColorAdded;
     bool mFragColorAdded;
     bool mFragDataAdded;
     bool mFragDepthAdded;
@@ -237,7 +241,7 @@ class CollectVariablesTraverser : public TIntermTraverser
     bool mPatchVerticesInAdded;
     bool mTessLevelOuterAdded;
     bool mTessLevelInnerAdded;
-    bool mBoundingBoxEXTAdded;
+    bool mBoundingBoxAdded;
     bool mTessCoordAdded;
     const int mTessControlShaderOutputVertices;
 
@@ -291,6 +295,7 @@ CollectVariablesTraverser::CollectVariablesTraverser(
       mHelperInvocationAdded(false),
       mFragCoordAdded(false),
       mLastFragDataAdded(false),
+      mLastFragColorAdded(false),
       mFragColorAdded(false),
       mFragDataAdded(false),
       mFragDepthAdded(false),
@@ -310,7 +315,7 @@ CollectVariablesTraverser::CollectVariablesTraverser(
       mPatchVerticesInAdded(false),
       mTessLevelOuterAdded(false),
       mTessLevelInnerAdded(false),
-      mBoundingBoxEXTAdded(false),
+      mBoundingBoxAdded(false),
       mTessCoordAdded(false),
       mTessControlShaderOutputVertices(tessControlShaderOutputVertices),
       mHashFunction(hashFunction),
@@ -336,7 +341,7 @@ void CollectVariablesTraverser::setBuiltInInfoFromSymbol(const TVariable &variab
         IsShaderIoBlock(type.getQualifier()) && type.getInterfaceBlock() != nullptr;
     bool isPatch = type.getQualifier() == EvqTessLevelInner ||
                    type.getQualifier() == EvqTessLevelOuter ||
-                   type.getQualifier() == EvqBoundingBoxEXT;
+                   type.getQualifier() == EvqBoundingBox;
 
     setFieldOrVariableProperties(type, true, isShaderIOBlock, isPatch, info);
 }
@@ -572,7 +577,7 @@ void CollectVariablesTraverser::visitSymbol(TIntermSymbol *symbol)
                 recordBuiltInAttributeUsed(symbol->variable(), &mLocalInvocationIndexAdded);
                 return;
             case EvqInstanceID:
-                // Whenever the SH_INITIALIZE_BUILTINS_FOR_INSTANCED_MULTIVIEW option is set,
+                // Whenever the initializeBuiltinsForInstancedMultiview option is set,
                 // gl_InstanceID is added inside expressions to initialize ViewID_OVR and
                 // InstanceID. Note that gl_InstanceID is not added to the symbol table for ESSL1
                 // shaders.
@@ -593,6 +598,9 @@ void CollectVariablesTraverser::visitSymbol(TIntermSymbol *symbol)
             case EvqLastFragData:
                 recordBuiltInVaryingUsed(symbol->variable(), &mLastFragDataAdded, mInputVaryings);
                 return;
+            case EvqLastFragColor:
+                recordBuiltInVaryingUsed(symbol->variable(), &mLastFragColorAdded, mInputVaryings);
+                return;
             case EvqFragColor:
                 recordBuiltInFragmentOutputUsed(symbol->variable(), &mFragColorAdded);
                 return;
@@ -629,15 +637,11 @@ void CollectVariablesTraverser::visitSymbol(TIntermSymbol *symbol)
                                              mInputVaryings);
                 }
                 break;
-            case EvqLayer:
+            case EvqLayerOut:
                 if (mShaderType == GL_GEOMETRY_SHADER_EXT)
                 {
                     recordBuiltInVaryingUsed(symbol->variable(), &mLayerAdded, mOutputVaryings);
                 }
-                else if (mShaderType == GL_FRAGMENT_SHADER)
-                {
-                    recordBuiltInVaryingUsed(symbol->variable(), &mLayerAdded, mInputVaryings);
-                }
                 else
                 {
                     ASSERT(mShaderType == GL_VERTEX_SHADER &&
@@ -645,6 +649,10 @@ void CollectVariablesTraverser::visitSymbol(TIntermSymbol *symbol)
                             IsExtensionEnabled(mExtensionBehavior, TExtension::OVR_multiview)));
                 }
                 break;
+            case EvqLayerIn:
+                ASSERT(mShaderType == GL_FRAGMENT_SHADER);
+                recordBuiltInVaryingUsed(symbol->variable(), &mLayerAdded, mInputVaryings);
+                break;
             case EvqShared:
                 if (mShaderType == GL_COMPUTE_SHADER)
                 {
@@ -707,9 +715,8 @@ void CollectVariablesTraverser::visitSymbol(TIntermSymbol *symbol)
                                              mInputVaryings);
                 }
                 break;
-            case EvqBoundingBoxEXT:
-                recordBuiltInVaryingUsed(symbol->variable(), &mBoundingBoxEXTAdded,
-                                         mOutputVaryings);
+            case EvqBoundingBox:
+                recordBuiltInVaryingUsed(symbol->variable(), &mBoundingBoxAdded, mOutputVaryings);
                 break;
             default:
                 break;
@@ -739,7 +746,9 @@ void CollectVariablesTraverser::setFieldOrVariableProperties(const TType &type,
     {
         // Structures use a NONE type that isn't exposed outside ANGLE.
         variableOut->type = GL_NONE;
-        if (structure->symbolType() != SymbolType::Empty)
+        // Anonymous structs are given name from AngleInternal namespace.
+        if (structure->symbolType() != SymbolType::Empty &&
+            structure->symbolType() != SymbolType::AngleInternal)
         {
             variableOut->structOrBlockName = structure->name().data();
         }
@@ -752,25 +761,28 @@ void CollectVariablesTraverser::setFieldOrVariableProperties(const TType &type,
             // ShaderVariable objects.
             ShaderVariable fieldVariable;
             setFieldProperties(*field->type(), field->name(), staticUse, isShaderIOBlock, isPatch,
-                               &fieldVariable);
+                               field->symbolType(), &fieldVariable);
             variableOut->fields.push_back(fieldVariable);
         }
     }
     else if (interfaceBlock && isShaderIOBlock)
     {
-        variableOut->type = GL_NONE;
+        const bool isPerVertex = (interfaceBlock->name() == "gl_PerVertex");
+        variableOut->type      = GL_NONE;
         if (interfaceBlock->symbolType() != SymbolType::Empty)
         {
             variableOut->structOrBlockName = interfaceBlock->name().data();
             variableOut->mappedStructOrBlockName =
-                HashName(interfaceBlock->name(), mHashFunction, nullptr).data();
+                isPerVertex ? interfaceBlock->name().data()
+                            : HashName(interfaceBlock->name(), mHashFunction, nullptr).data();
         }
         const TFieldList &fields = interfaceBlock->fields();
         for (const TField *field : fields)
         {
             ShaderVariable fieldVariable;
+
             setFieldProperties(*field->type(), field->name(), staticUse, true, isPatch,
-                               &fieldVariable);
+                               field->symbolType(), &fieldVariable);
             fieldVariable.isShaderIOBlock = true;
             variableOut->fields.push_back(fieldVariable);
         }
@@ -781,7 +793,7 @@ void CollectVariablesTraverser::setFieldOrVariableProperties(const TType &type,
         variableOut->precision = GLVariablePrecision(type);
     }
 
-    const TSpan<const unsigned int> &arraySizes = type.getArraySizes();
+    const angle::Span<const unsigned int> &arraySizes = type.getArraySizes();
     if (!arraySizes.empty())
     {
         variableOut->arraySizes.assign(arraySizes.begin(), arraySizes.end());
@@ -814,12 +826,15 @@ void CollectVariablesTraverser::setFieldProperties(const TType &type,
                                                    bool staticUse,
                                                    bool isShaderIOBlock,
                                                    bool isPatch,
+                                                   SymbolType symbolType,
                                                    ShaderVariable *variableOut) const
 {
     ASSERT(variableOut);
     setFieldOrVariableProperties(type, staticUse, isShaderIOBlock, isPatch, variableOut);
     variableOut->name.assign(name.data(), name.length());
-    variableOut->mappedName = HashName(name, mHashFunction, nullptr).data();
+    variableOut->mappedName = (symbolType == SymbolType::BuiltIn)
+                                  ? name.data()
+                                  : HashName(name, mHashFunction, nullptr).data();
 }
 
 void CollectVariablesTraverser::setCommonVariableProperties(const TType &type,
@@ -903,8 +918,10 @@ ShaderVariable CollectVariablesTraverser::recordVarying(const TIntermSymbol &var
         case EvqFlatOut:
         case EvqNoPerspectiveOut:
         case EvqCentroidOut:
-        case EvqGeometryOut:
         case EvqSampleOut:
+        case EvqNoPerspectiveCentroidOut:
+        case EvqNoPerspectiveSampleOut:
+        case EvqGeometryOut:
             if (mSymbolTable->isVaryingInvariant(variable.variable()) || type.isInvariant())
             {
                 varying.isInvariant = true;
@@ -1004,8 +1021,8 @@ void CollectVariablesTraverser::recordInterfaceBlock(const char *instanceName,
         interfaceBlockType.isArray() ? interfaceBlockType.getOutermostArraySize() : 0;
 
     interfaceBlock->blockType = GetBlockType(interfaceBlockType.getQualifier());
-    if (interfaceBlock->blockType == BlockType::BLOCK_UNIFORM ||
-        interfaceBlock->blockType == BlockType::BLOCK_BUFFER)
+    if (interfaceBlock->blockType == BlockType::kBlockUniform ||
+        interfaceBlock->blockType == BlockType::kBlockBuffer)
     {
         // TODO(oetuaho): Remove setting isRowMajorLayout.
         interfaceBlock->isRowMajorLayout = false;
@@ -1013,6 +1030,12 @@ void CollectVariablesTraverser::recordInterfaceBlock(const char *instanceName,
         interfaceBlock->layout           = GetBlockLayoutType(blockType->blockStorage());
     }
 
+    // Consider an SSBO readonly if all its fields are readonly.  Note that ANGLE doesn't keep the
+    // readonly qualifier applied to the interface block itself, but rather applies it to the
+    // fields.
+    ASSERT(!interfaceBlockType.getMemoryQualifier().readonly);
+    bool isReadOnly = true;
+
     // Gather field information
     bool anyFieldStaticallyUsed = false;
 
@@ -1036,15 +1059,26 @@ void CollectVariablesTraverser::recordInterfaceBlock(const char *instanceName,
         }
 
         ShaderVariable fieldVariable;
-        setFieldProperties(fieldType, field->name(), staticUse, false, false, &fieldVariable);
+        setFieldProperties(fieldType, field->name(), staticUse, false, false, field->symbolType(),
+                           &fieldVariable);
         fieldVariable.isRowMajorLayout =
             (fieldType.getLayoutQualifier().matrixPacking == EmpRowMajor);
         interfaceBlock->fields.push_back(fieldVariable);
+
+        // The SSBO is not readonly if any field is not readonly.
+        if (!fieldType.getMemoryQualifier().readonly)
+        {
+            isReadOnly = false;
+        }
     }
     if (anyFieldStaticallyUsed)
     {
         interfaceBlock->staticUse = true;
     }
+    if (interfaceBlock->blockType == BlockType::kBlockBuffer)
+    {
+        interfaceBlock->isReadOnly = isReadOnly;
+    }
 }
 
 ShaderVariable CollectVariablesTraverser::recordUniform(const TIntermSymbol &variable) const
@@ -1054,10 +1088,11 @@ ShaderVariable CollectVariablesTraverser::recordUniform(const TIntermSymbol &var
     uniform.binding = variable.getType().getLayoutQualifier().binding;
     uniform.imageUnitFormat =
         GetImageInternalFormatType(variable.getType().getLayoutQualifier().imageInternalFormat);
-    uniform.location  = variable.getType().getLayoutQualifier().location;
-    uniform.offset    = variable.getType().getLayoutQualifier().offset;
-    uniform.readonly  = variable.getType().getMemoryQualifier().readonly;
-    uniform.writeonly = variable.getType().getMemoryQualifier().writeonly;
+    uniform.location      = variable.getType().getLayoutQualifier().location;
+    uniform.offset        = variable.getType().getLayoutQualifier().offset;
+    uniform.rasterOrdered = variable.getType().getLayoutQualifier().rasterOrdered;
+    uniform.readonly      = variable.getType().getMemoryQualifier().readonly;
+    uniform.writeonly     = variable.getType().getMemoryQualifier().writeonly;
     return uniform;
 }
 
@@ -1114,6 +1149,10 @@ bool CollectVariablesTraverser::visitDeclaration(Visit, TIntermDeclaration *node
                 case EvqBuffer:
                     mShaderStorageBlocks->push_back(interfaceBlock);
                     break;
+                case EvqPixelLocalEXT:
+                    // EXT_shader_pixel_local_storage is completely self-contained within the
+                    // shader, so we don't need to gather any info on it.
+                    break;
                 default:
                     UNREACHABLE();
             }
@@ -1136,6 +1175,9 @@ bool CollectVariablesTraverser::visitDeclaration(Visit, TIntermDeclaration *node
                 case EvqUniform:
                     mUniforms->push_back(recordUniform(variable));
                     break;
+                case EvqPerVertexIn:
+                case EvqPerVertexOut:
+                    break;
                 default:
                     if (IsVaryingIn(qualifier))
                     {
@@ -1223,8 +1265,9 @@ bool CollectVariablesTraverser::visitBinary(Visit, TIntermBinary *binaryNode)
         {
             MarkActive(ioBlockVar);
         }
-        else
+        else if (qualifier != EvqPixelLocalEXT)
         {
+
             if (!namedBlock)
             {
                 namedBlock = findNamedInterfaceBlock(interfaceBlock->name());
@@ -1236,8 +1279,8 @@ bool CollectVariablesTraverser::visitBinary(Visit, TIntermBinary *binaryNode)
             ASSERT(fieldIndex < namedBlock->fields.size());
             // TODO(oetuaho): Would be nicer to record static use of fields of named interface
             // blocks more accurately at parse time - now we only mark the fields statically used if
-            // they are active. http://anglebug.com/2440 We need to mark this field and all of its
-            // sub-fields, as static/active
+            // they are active. http://anglebug.com/42261150 We need to mark this field and all of
+            // its sub-fields, as static/active
             MarkActive(&namedBlock->fields[fieldIndex]);
         }
 
