@@ -1,4 +1,5 @@
 //
+//
 // Copyright 2002 The ANGLE Project Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
@@ -17,13 +18,10 @@
 #include "angle_gl.h"
 #include "common/MemoryBuffer.h"
 #include "common/PackedEnums.h"
+#include "common/SimpleMutex.h"
 #include "common/angleutils.h"
 #include "libANGLE/Caps.h"
 #include "libANGLE/Constants.h"
-#include "libANGLE/Context_gl_1_autogen.h"
-#include "libANGLE/Context_gl_2_autogen.h"
-#include "libANGLE/Context_gl_3_autogen.h"
-#include "libANGLE/Context_gl_4_autogen.h"
 #include "libANGLE/Context_gles_1_0_autogen.h"
 #include "libANGLE/Context_gles_2_0_autogen.h"
 #include "libANGLE/Context_gles_3_0_autogen.h"
@@ -31,20 +29,22 @@
 #include "libANGLE/Context_gles_3_2_autogen.h"
 #include "libANGLE/Context_gles_ext_autogen.h"
 #include "libANGLE/Error.h"
+#include "libANGLE/Framebuffer.h"
 #include "libANGLE/HandleAllocator.h"
 #include "libANGLE/RefCountObject.h"
 #include "libANGLE/ResourceManager.h"
 #include "libANGLE/ResourceMap.h"
 #include "libANGLE/State.h"
 #include "libANGLE/VertexAttribute.h"
-#include "libANGLE/WorkerThread.h"
 #include "libANGLE/angletypes.h"
 
 namespace angle
 {
+class Closure;
 class FrameCapture;
 class FrameCaptureShared;
 struct FrontendFeatures;
+class WaitableEvent;
 }  // namespace angle
 
 namespace rx
@@ -66,10 +66,11 @@ namespace gl
 class Buffer;
 class Compiler;
 class FenceNV;
-class Framebuffer;
 class GLES1Renderer;
 class MemoryProgramCache;
+class MemoryShaderCache;
 class MemoryObject;
+class PixelLocalStoragePlane;
 class Program;
 class ProgramPipeline;
 class Query;
@@ -86,10 +87,12 @@ struct VertexAttribute;
 class ErrorSet : angle::NonCopyable
 {
   public:
-    explicit ErrorSet(Context *context);
+    explicit ErrorSet(Debug *debug,
+                      const angle::FrontendFeatures &frontendFeatures,
+                      const egl::AttributeMap &attribs);
     ~ErrorSet();
 
-    bool empty() const;
+    bool empty() const { return mHasAnyErrors.load(std::memory_order_relaxed) == 0; }
     GLenum popError();
 
     void handleError(GLenum errorCode,
@@ -99,10 +102,58 @@ class ErrorSet : angle::NonCopyable
                      unsigned int line);
 
     void validationError(angle::EntryPoint entryPoint, GLenum errorCode, const char *message);
+    ANGLE_FORMAT_PRINTF(4, 5)
+    void validationErrorF(angle::EntryPoint entryPoint, GLenum errorCode, const char *format, ...);
+
+    bool skipValidation() const
+    {
+        // Ensure we don't skip validation when context becomes lost, since implementations
+        // generally assume a non-lost context, non-null objects, etc.
+        ASSERT(!isContextLost() || !mSkipValidation);
+        return mSkipValidation.load(std::memory_order_relaxed) != 0;
+    }
+    void forceValidation() { mSkipValidation = 0; }
+
+    void markContextLost(GraphicsResetStatus status);
+    bool isContextLost() const { return mContextLost.load(std::memory_order_relaxed) != 0; }
+    GLenum getGraphicsResetStatus(rx::ContextImpl *contextImpl);
+    GLenum getResetStrategy() const { return mResetStrategy; }
+    GLenum getErrorForCapture() const;
+#if defined(ANGLE_ENABLE_ASSERTS)
+    uint32_t getPushedErrorCount() const { return mPushedErrors; }
+#endif
 
   private:
-    Context *mContext;
+    void setContextLost();
+    void pushError(GLenum errorCode);
+    std::unique_lock<std::mutex> getLockIfNotAlready();
+
+    // Non-atomic members of this class are protected by a mutex.  This is to allow errors to be
+    // safely set by entry points that don't hold a lock.  Note that other contexts may end up
+    // triggering an error on this context (through making failable calls on other contexts in the
+    // share group).
+    //
+    // Note also that the functionality used through the Debug class is thread-safe.
+    std::mutex mMutex;
+
+    // Error handling and reporting
+    Debug *mDebug;
     std::set<GLenum> mErrors;
+
+    const GLenum mResetStrategy;
+    const bool mLoseContextOnOutOfMemory;
+
+    // Context-loss handling
+    bool mContextLostForced;
+    GraphicsResetStatus mResetStatus;
+
+    // The following are atomic and lockless as they are very frequently accessed.
+    std::atomic_int mSkipValidation;
+    std::atomic_int mContextLost;
+#if defined(ANGLE_ENABLE_ASSERTS)
+    std::atomic_uint32_t mPushedErrors;  // must be unsigned to handle overflows
+#endif
+    std::atomic_int mHasAnyErrors;
 };
 
 enum class VertexAttribTypeCase
@@ -113,6 +164,52 @@ enum class VertexAttribTypeCase
     ValidSize3or4  = 3,
 };
 
+// Part of StateCache (see below) that is private to the context and is inaccessible to other
+// contexts.
+class PrivateStateCache final : angle::NonCopyable
+{
+  public:
+    PrivateStateCache();
+    ~PrivateStateCache();
+
+    void onCapChange() { mIsCachedBasicDrawStatesErrorValid = false; }
+    void onColorMaskChange() { mIsCachedBasicDrawStatesErrorValid = false; }
+    void onDefaultVertexAttributeChange() { mIsCachedBasicDrawStatesErrorValid = false; }
+
+    // Blending updates invalidate draw
+    // state in the following cases:
+    //
+    // * Blend equations have been changed and the context
+    //   supports KHR_blend_equation_advanced. The number
+    //   of enabled draw buffers may need to be checked
+    //   to not be greater than 1.
+    //
+    // * Blend funcs have been changed with indexed
+    //   commands. The D3D11 backend cannot support
+    //   constant color and alpha blend funcs together
+    //   so a check is needed across all draw buffers.
+    //
+    // * Blend funcs have been changed and the context
+    //   supports EXT_blend_func_extended. The number
+    //   of enabled draw buffers may need to be checked
+    //   against MAX_DUAL_SOURCE_DRAW_BUFFERS_EXT limit.
+    void onBlendEquationOrFuncChange() { mIsCachedBasicDrawStatesErrorValid = false; }
+
+    void onStencilStateChange() { mIsCachedBasicDrawStatesErrorValid = false; }
+
+    bool isCachedBasicDrawStatesErrorValid() const { return mIsCachedBasicDrawStatesErrorValid; }
+    void setCachedBasicDrawStatesErrorValid() const { mIsCachedBasicDrawStatesErrorValid = true; }
+
+  private:
+    // StateCache::mCachedBasicDrawStatesError* may be invalidated through numerous calls (see the
+    // comment on getBasicDrawStatesErrorString), some of which may originate from other contexts
+    // (through the observer interface).  However, ContextPrivate* helpers may also need to
+    // invalidate the draw states, but they are called without holding the share group lock.  The
+    // following tracks whether StateCache::mCachedBasicDrawStatesError* values are valid and is
+    // accessed only by the context itself.
+    mutable bool mIsCachedBasicDrawStatesErrorValid;
+};
+
 // Helper class for managing cache variables and state changes.
 class StateCache final : angle::NonCopyable
 {
@@ -127,6 +224,7 @@ class StateCache final : angle::NonCopyable
     // 2. onProgramExecutableChange.
     // 3. onVertexArrayStateChange.
     // 4. onGLES1ClientStateChange.
+    // 5. onGLES1TextureStateChange.
     AttributesMask getActiveBufferedAttribsMask() const { return mCachedActiveBufferedAttribsMask; }
     AttributesMask getActiveClientAttribsMask() const { return mCachedActiveClientAttribsMask; }
     AttributesMask getActiveDefaultAttribsMask() const { return mCachedActiveDefaultAttribsMask; }
@@ -152,37 +250,51 @@ class StateCache final : angle::NonCopyable
     // 4. onVertexArrayStateChange.
     // 5. onVertexArrayBufferStateChange.
     // 6. onDrawFramebufferChange.
-    // 7. onContextCapChange.
-    // 8. onStencilStateChange.
-    // 9. onDefaultVertexAttributeChange.
-    // 10. onActiveTextureChange.
-    // 11. onQueryChange.
-    // 12. onActiveTransformFeedbackChange.
-    // 13. onUniformBufferStateChange.
-    // 14. onColorMaskChange.
-    // 15. onBufferBindingChange.
-    // 16. onBlendFuncIndexedChange.
-    bool hasBasicDrawStatesError(Context *context) const
+    // 7. onActiveTextureChange.
+    // 8. onQueryChange.
+    // 9. onActiveTransformFeedbackChange.
+    // 10. onUniformBufferStateChange.
+    // 11. onBufferBindingChange.
+    //
+    // Additionally, the following in PrivateStateCache can lead to updateBasicDrawStatesError:
+    // 1. onCapChange.
+    // 2. onStencilStateChange.
+    // 3. onDefaultVertexAttributeChange.
+    // 4. onColorMaskChange.
+    // 5. onBlendEquationOrFuncChange.
+    intptr_t getBasicDrawStatesErrorString(const Context *context,
+                                           const PrivateStateCache *privateStateCache) const
     {
-        if (mCachedBasicDrawStatesError == 0)
+        // This is only ever called with the context that owns this state cache
+        ASSERT(isCurrentContext(context, privateStateCache));
+        if (privateStateCache->isCachedBasicDrawStatesErrorValid() &&
+            mCachedBasicDrawStatesErrorString != kInvalidPointer)
         {
-            return false;
+            return mCachedBasicDrawStatesErrorString;
         }
-        if (mCachedBasicDrawStatesError != kInvalidPointer)
-        {
-            return true;
-        }
-        return getBasicDrawStatesErrorImpl(context) != 0;
+
+        return getBasicDrawStatesErrorImpl(context, privateStateCache);
     }
 
-    intptr_t getBasicDrawStatesError(const Context *context) const
+    // The GL error enum to use when generating errors due to failed draw states. Only valid if
+    // getBasicDrawStatesErrorString returns non-zero.
+    GLenum getBasicDrawElementsErrorCode() const
     {
-        if (mCachedBasicDrawStatesError != kInvalidPointer)
+        ASSERT(mCachedBasicDrawStatesErrorString != kInvalidPointer);
+        ASSERT(mCachedBasicDrawStatesErrorCode != GL_NO_ERROR);
+        return mCachedBasicDrawStatesErrorCode;
+    }
+
+    // Places that can trigger updateProgramPipelineError:
+    // 1. onProgramExecutableChange.
+    intptr_t getProgramPipelineError(const Context *context) const
+    {
+        if (mCachedProgramPipelineError != kInvalidPointer)
         {
-            return mCachedBasicDrawStatesError;
+            return mCachedProgramPipelineError;
         }
 
-        return getBasicDrawStatesErrorImpl(context);
+        return getProgramPipelineErrorImpl(context);
     }
 
     // Places that can trigger updateBasicDrawElementsError:
@@ -261,23 +373,20 @@ class StateCache final : angle::NonCopyable
     void onVertexArrayBufferContentsChange(Context *context);
     void onVertexArrayStateChange(Context *context);
     void onVertexArrayBufferStateChange(Context *context);
+    void onGLES1TextureStateChange(Context *context);
     void onGLES1ClientStateChange(Context *context);
     void onDrawFramebufferChange(Context *context);
-    void onContextCapChange(Context *context);
-    void onStencilStateChange(Context *context);
-    void onDefaultVertexAttributeChange(Context *context);
     void onActiveTextureChange(Context *context);
     void onQueryChange(Context *context);
     void onActiveTransformFeedbackChange(Context *context);
     void onUniformBufferStateChange(Context *context);
     void onAtomicCounterBufferStateChange(Context *context);
     void onShaderStorageBufferStateChange(Context *context);
-    void onColorMaskChange(Context *context);
     void onBufferBindingChange(Context *context);
-    void onBlendFuncIndexedChange(Context *context);
-    void onBlendEquationChange(Context *context);
 
   private:
+    bool isCurrentContext(const Context *context, const PrivateStateCache *privateStateCache) const;
+
     // Cache update functions.
     void updateActiveAttribsMask(Context *context);
     void updateVertexElementLimits(Context *context);
@@ -285,8 +394,13 @@ class StateCache final : angle::NonCopyable
     void updateValidDrawModes(Context *context);
     void updateValidBindTextureTypes(Context *context);
     void updateValidDrawElementsTypes(Context *context);
-    void updateBasicDrawStatesError();
-    void updateBasicDrawElementsError();
+    void updateBasicDrawStatesError()
+    {
+        mCachedBasicDrawStatesErrorString = kInvalidPointer;
+        mCachedBasicDrawStatesErrorCode   = GL_NO_ERROR;
+    }
+    void updateProgramPipelineError() { mCachedProgramPipelineError = kInvalidPointer; }
+    void updateBasicDrawElementsError() { mCachedBasicDrawElementsError = kInvalidPointer; }
     void updateTransformFeedbackActiveUnpaused(Context *context);
     void updateVertexAttribTypesValidation(Context *context);
     void updateActiveShaderStorageBufferIndices(Context *context);
@@ -300,7 +414,9 @@ class StateCache final : angle::NonCopyable
                            bool triAdjOK,
                            bool patchOK);
 
-    intptr_t getBasicDrawStatesErrorImpl(const Context *context) const;
+    intptr_t getBasicDrawStatesErrorImpl(const Context *context,
+                                         const PrivateStateCache *privateStateCache) const;
+    intptr_t getProgramPipelineErrorImpl(const Context *context) const;
     intptr_t getBasicDrawElementsErrorImpl(const Context *context) const;
 
     static constexpr intptr_t kInvalidPointer = 1;
@@ -308,11 +424,66 @@ class StateCache final : angle::NonCopyable
     AttributesMask mCachedActiveBufferedAttribsMask;
     AttributesMask mCachedActiveClientAttribsMask;
     AttributesMask mCachedActiveDefaultAttribsMask;
-    bool mCachedHasAnyEnabledClientAttrib;
+
+    // Given a vertex attribute's stride, the corresponding vertex buffer can fit a number of such
+    // attributes.  A draw call that attempts to use more vertex attributes thus needs to fail (when
+    // robust access is enabled).  The following variables help implement this limit given the
+    // following situations:
+    //
+    // Assume:
+    //
+    // Ni = Number of vertex attributes that can fit in buffer bound to attribute i.
+    // Di = Vertex attribute divisor set for attribute i.
+    // F = Draw calls "first" vertex index
+    // C = Draw calls vertex "count"
+    // B = Instanced draw calls "baseinstance"
+    // P = Instanced draw calls "primcount"
+    //
+    // Then, for each attribute i:
+    //
+    //   If Di == 0 (i.e. non-instanced)
+    //     Vertices [F, F+C) are accessed
+    //     Draw call should fail if F+C > Ni
+    //
+    //   If Di != 0 (i.e. instanced), in a non-instanced draw call:
+    //     Only vertex 0 is accessed - note that a non-zero divisor in a non-instanced draw call
+    //       implies that F is ignored and the vertex index is not incremented.
+    //     Draw call should fail if Ni < 1
+    //
+    //   If Di != 0, in an instanced draw call:
+    //     Vertices [B, B+ceil(P/Di)) are accessed
+    //     Draw call should fail if B+ceil(P/Di) > Ni
+    //
+    // To avoid needing to iterate over all attributes in the hot paths, the following is
+    // calculated:
+    //
+    // Non-instanced limit: min(Ni) for all non-instanced attributes.  At draw time F+C <= min(Ni)
+    // is validated.
+    // Instanced limit: min(Ni*Di) for all instanced attributes.  At draw time, B+P <= min(Ni*Di) is
+    // validated (the math works out, try with an example!)
+    //
+    // For instanced attributes in a non-instanced draw call, need to check that min(Ni) > 0.
+    // Evaluating min(Ni*DI) > 0 produces the same result though, so the instanced limit is used
+    // there too.
+    //
+    // If there are no instanced attributes, the non-instanced limit is set to infinity.  If there
+    // are no instanced attributes, the instanced limits are set to infinity.
     GLint64 mCachedNonInstancedVertexElementLimit;
     GLint64 mCachedInstancedVertexElementLimit;
-    mutable intptr_t mCachedBasicDrawStatesError;
+
+    mutable intptr_t mCachedBasicDrawStatesErrorString;
+    mutable GLenum mCachedBasicDrawStatesErrorCode;
     mutable intptr_t mCachedBasicDrawElementsError;
+    // mCachedProgramPipelineError checks only the
+    // current-program-exists subset of mCachedBasicDrawStatesError.
+    // Therefore, mCachedProgramPipelineError follows
+    // mCachedBasicDrawStatesError in that if mCachedBasicDrawStatesError is
+    // no-error, so is mCachedProgramPipelineError.  Otherwise, if
+    // mCachedBasicDrawStatesError is in error, the state of
+    // mCachedProgramPipelineError can be no-error or also in error, or
+    // unknown due to early exiting.
+    mutable intptr_t mCachedProgramPipelineError;
+    bool mCachedHasAnyEnabledClientAttrib;
     bool mCachedTransformFeedbackActiveUnpaused;
     StorageBuffersMask mCachedActiveShaderStorageBufferIndices;
     ImageUnitMask mCachedActiveImageUnitIndices;
@@ -348,8 +519,9 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
             const Context *shareContext,
             TextureManager *shareTextures,
             SemaphoreManager *shareSemaphores,
+            egl::ContextMutex *sharedContextMutex,
             MemoryProgramCache *memoryProgramCache,
-            const EGLenum clientType,
+            MemoryShaderCache *memoryShaderCache,
             const egl::AttributeMap &attribs,
             const egl::DisplayExtensions &displayExtensions,
             const egl::ClientExtensions &clientExtensions);
@@ -390,7 +562,7 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
 
     Buffer *getBuffer(BufferID handle) const;
     FenceNV *getFenceNV(FenceNVID handle) const;
-    Sync *getSync(GLsync handle) const;
+    Sync *getSync(SyncID syncPacked) const;
     ANGLE_INLINE Texture *getTexture(TextureID handle) const
     {
         return mState.mTextureManager->getTexture(handle);
@@ -416,8 +588,9 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     bool isVertexArrayGenerated(VertexArrayID vertexArray) const;
     bool isTransformFeedbackGenerated(TransformFeedbackID transformFeedback) const;
 
-    bool isExternal() const { return mIsExternal; }
-    bool saveAndRestoreState() const { return mSaveAndRestoreState; }
+    bool isZeroTextureBound(TextureType textureType) const;
+
+    bool isExternal() const { return mState.isExternal(); }
 
     void getBooleanvImpl(GLenum pname, GLboolean *params) const;
     void getFloatvImpl(GLenum pname, GLfloat *params) const;
@@ -432,12 +605,6 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
 
     bool hasActiveTransformFeedback(ShaderProgramID program) const;
 
-    // Desktop GL entry point interface
-    ANGLE_GL_1_CONTEXT_API
-    ANGLE_GL_2_CONTEXT_API
-    ANGLE_GL_3_CONTEXT_API
-    ANGLE_GL_4_CONTEXT_API
-
     // GLES entry point interface
     ANGLE_GLES_1_0_CONTEXT_API
     ANGLE_GLES_2_0_CONTEXT_API
@@ -455,23 +622,11 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
                      const char *function,
                      unsigned int line);
 
-    void validationError(angle::EntryPoint entryPoint, GLenum errorCode, const char *message) const;
-    ANGLE_FORMAT_PRINTF(4, 5)
-    void validationErrorF(angle::EntryPoint entryPoint,
-                          GLenum errorCode,
-                          const char *format,
-                          ...) const;
-
-    void markContextLost(GraphicsResetStatus status);
-
-    bool isContextLost() const { return mContextLost; }
-    void setContextLost();
-
-    GLenum getGraphicsResetStrategy() const { return mResetStrategy; }
     bool isResetNotificationEnabled() const;
 
-    const egl::Config *getConfig() const;
-    EGLenum getClientType() const;
+    bool isRobustnessEnabled() const { return mState.hasRobustAccess(); }
+
+    const egl::Config *getConfig() const { return mConfig; }
     EGLenum getRenderBuffer() const;
     EGLenum getContextPriority() const;
 
@@ -499,7 +654,9 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     angle::Result prepareForInvalidate(GLenum target);
 
     MemoryProgramCache *getMemoryProgramCache() const { return mMemoryProgramCache; }
-    std::mutex &getProgramCacheMutex() const;
+    MemoryShaderCache *getMemoryShaderCache() const { return mMemoryShaderCache; }
+
+    angle::SimpleMutex &getProgramCacheMutex() const;
 
     bool hasBeenCurrent() const { return mHasBeenCurrent; }
     egl::Display *getDisplay() const { return mDisplay; }
@@ -520,8 +677,7 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     void setShared() { mShared = true; }
 
     const State &getState() const { return mState; }
-    GLint getClientMajorVersion() const { return mState.getClientMajorVersion(); }
-    GLint getClientMinorVersion() const { return mState.getClientMinorVersion(); }
+    const PrivateState &getPrivateState() const { return mState.privateState(); }
     const Version &getClientVersion() const { return mState.getClientVersion(); }
     const Caps &getCaps() const { return mState.getCaps(); }
     const TextureCapsMap &getTextureCaps() const { return mState.getTextureCaps(); }
@@ -529,13 +685,19 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     const Limitations &getLimitations() const { return mState.getLimitations(); }
     bool isGLES1() const;
 
-    bool skipValidation() const
-    {
-        // Ensure we don't skip validation when context becomes lost, since implementations
-        // generally assume a non-lost context, non-null objects, etc.
-        ASSERT(!isContextLost() || !mSkipValidation);
-        return mSkipValidation;
-    }
+    // To be used **only** directly by the entry points.
+    PrivateState *getMutablePrivateState() { return mState.getMutablePrivateState(); }
+    GLES1State *getMutableGLES1State() { return mState.getMutableGLES1State(); }
+
+    bool skipValidation() const { return mErrors.skipValidation(); }
+    void markContextLost(GraphicsResetStatus status) { mErrors.markContextLost(status); }
+    bool isContextLost() const { return mErrors.isContextLost(); }
+
+    // Some commands may need to generate a context lost error but still return a value.
+    // The validation layer does not generate the context lost error in such cases.
+    void contextLostErrorOnBlockingCall(angle::EntryPoint entryPoint) const;
+
+    ErrorSet *getMutableErrorSetForValidation() const { return &mErrors; }
 
     // Specific methods needed for validation.
     bool getQueryParameterInfo(GLenum pname, GLenum *type, unsigned int *numParams) const;
@@ -544,7 +706,7 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     ANGLE_INLINE Program *getProgramResolveLink(ShaderProgramID handle) const
     {
         Program *program = mState.mShaderProgramManager->getProgram(handle);
-        if (program)
+        if (ANGLE_LIKELY(program))
         {
             program->resolveLink(this);
         }
@@ -552,7 +714,14 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     }
 
     Program *getProgramNoResolveLink(ShaderProgramID handle) const;
-    Shader *getShader(ShaderProgramID handle) const;
+    Shader *getShaderResolveCompile(ShaderProgramID handle) const;
+    Shader *getShaderNoResolveCompile(ShaderProgramID handle) const;
+
+    bool nameStartsWithReservedPrefix(const GLchar *name) const
+    {
+        return (strncmp(name, "gl_", 3) == 0) ||
+               (isWebGL() && (strncmp(name, "webgl_", 6) == 0 || strncmp(name, "_webgl_", 7) == 0));
+    }
 
     ANGLE_INLINE bool isTextureGenerated(TextureID texture) const
     {
@@ -577,6 +746,7 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
 
     bool isWebGL() const { return mState.isWebGL(); }
     bool isWebGL1() const { return mState.isWebGL1(); }
+    const char *getRendererString() const { return mRendererString; }
 
     bool isValidBufferBinding(BufferBinding binding) const { return mValidBufferBindings[binding]; }
 
@@ -585,24 +755,25 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     static int TexCoordArrayIndex(unsigned int unit);
 
     // GL_KHR_parallel_shader_compile
-    std::shared_ptr<angle::WorkerThreadPool> getShaderCompileThreadPool() const
-    {
-        if (mState.mExtensions.parallelShaderCompileKHR)
-        {
-            return mMultiThreadPool;
-        }
-        return mSingleThreadPool;
-    }
+    std::shared_ptr<angle::WorkerThreadPool> getShaderCompileThreadPool() const;
+    std::shared_ptr<angle::WorkerThreadPool> getLinkSubTaskThreadPool() const;
+    std::shared_ptr<angle::WaitableEvent> postCompileLinkTask(
+        const std::shared_ptr<angle::Closure> &task,
+        angle::JobThreadSafety safety,
+        angle::JobResultExpectancy resultExpectancy) const;
+
+    // Single-threaded pool; runs everything instantly
+    std::shared_ptr<angle::WorkerThreadPool> getSingleThreadPool() const;
 
     // Generic multithread pool.
-    std::shared_ptr<angle::WorkerThreadPool> getWorkerThreadPool() const
-    {
-        return mMultiThreadPool;
-    }
+    std::shared_ptr<angle::WorkerThreadPool> getWorkerThreadPool() const;
 
     const StateCache &getStateCache() const { return mStateCache; }
     StateCache &getStateCache() { return mStateCache; }
 
+    const PrivateStateCache &getPrivateStateCache() const { return mPrivateStateCache; }
+    PrivateStateCache *getMutablePrivateStateCache() { return &mPrivateStateCache; }
+
     void onSubjectStateChange(angle::SubjectIndex index, angle::SubjectMessage message) override;
 
     void onSamplerUniformChange(size_t textureUnitIndex);
@@ -619,63 +790,118 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     {
         return mTransformFeedbackMap;
     }
+    GLenum getErrorForCapture() const { return mErrors.getErrorForCapture(); }
+#if defined(ANGLE_ENABLE_ASSERTS)
+    uint32_t getPushedErrorCount() const { return mErrors.getPushedErrorCount(); }
+#endif
 
-    void onPreSwap() const;
+    void onPreSwap();
 
-    Program *getActiveLinkedProgram() const;
+    ANGLE_INLINE Program *getActiveLinkedProgram() const
+    {
+        Program *program = mState.getLinkedProgram(this);
+        if (ANGLE_LIKELY(program))
+        {
+            return program;
+        }
+        return getActiveLinkedProgramPPO();
+    }
+
+    ANGLE_NOINLINE Program *getActiveLinkedProgramPPO() const;
 
     // EGL_ANGLE_power_preference implementation.
     egl::Error releaseHighPowerGPU();
     egl::Error reacquireHighPowerGPU();
     void onGPUSwitch();
 
+    // EGL_ANGLE_external_context_and_surface implementation.
+    egl::Error acquireExternalContext(egl::Surface *drawAndReadSurface);
+    egl::Error releaseExternalContext();
+
+    bool noopDrawProgram() const;
     bool noopDraw(PrimitiveMode mode, GLsizei count) const;
     bool noopDrawInstanced(PrimitiveMode mode, GLsizei count, GLsizei instanceCount) const;
     bool noopMultiDraw(GLsizei drawcount) const;
 
-    bool isClearBufferMaskedOut(GLenum buffer, GLint drawbuffer) const;
+    bool isClearBufferMaskedOut(GLenum buffer,
+                                GLint drawbuffer,
+                                GLuint framebufferStencilSize) const;
     bool noopClearBuffer(GLenum buffer, GLint drawbuffer) const;
 
     void addRef() const { mRefCount++; }
     void release() const { mRefCount--; }
-    size_t getRefCount() const { return mRefCount; }
+    bool isReferenced() const { return mRefCount > 0; }
 
     egl::ShareGroup *getShareGroup() const { return mState.getShareGroup(); }
 
+    // Warning! When need to store pointer to the mutex in other object use `getRoot()` pointer, do
+    // NOT get pointer of the `getContextMutex()` reference.
+    egl::ContextMutex &getContextMutex() const { return mState.mContextMutex; }
+
     bool supportsGeometryOrTesselation() const;
     void dirtyAllState();
 
     bool isDestroyed() const { return mIsDestroyed; }
     void setIsDestroyed() { mIsDestroyed = true; }
 
+    // This function acts as glEnable(GL_COLOR_LOGIC_OP), but it's called from the GLES1 emulation
+    // code to implement logicOp using the non-GLES1 functionality (i.e. GL_ANGLE_logic_op).  The
+    // ContextPrivateEnable() entry point implementation cannot be used (as ContextPrivate*
+    // functions are typically used by other frontend-emulated features) because it forwards this
+    // back to GLES1.
+    void setLogicOpEnabledForGLES1(bool enabled);
+
     // Needed by capture serialization logic that works with a "const" Context pointer.
     void finishImmutable() const;
 
     const angle::PerfMonitorCounterGroups &getPerfMonitorCounterGroups() const;
 
+    // Ends the currently active pixel local storage session with GL_STORE_OP_STORE on all planes.
+    void endPixelLocalStorageImplicit();
+
+    bool areBlobCacheFuncsSet() const;
+
+    size_t getMemoryUsage() const;
+
+    // Only used by vulkan backend.
+    void onSwapChainImageChanged() const { mDefaultFramebuffer->onSwapChainImageChanged(); }
+    void onBufferChanged(const angle::SubjectMessage message,
+                         VertexArrayBufferBindingMask vertexArrayBufferBindingMask) const
+    {
+        // Notify current vertex array of the buffer changed. Note that other vertex arrays of this
+        // context or other context requires rebind which will check buffer changes
+        // at that time.
+        if (vertexArrayBufferBindingMask.any())
+        {
+            ASSERT(mState.mVertexArray != nullptr);
+            mState.mVertexArray->onBufferChanged(this, message, vertexArrayBufferBindingMask);
+        }
+    }
+
   private:
     void initializeDefaultResources();
+    void releaseSharedObjects();
 
     angle::Result prepareForDraw(PrimitiveMode mode);
     angle::Result prepareForClear(GLbitfield mask);
     angle::Result prepareForClearBuffer(GLenum buffer, GLint drawbuffer);
-    angle::Result syncState(const State::DirtyBits &bitMask,
-                            const State::DirtyObjects &objectMask,
+    angle::Result syncState(const state::DirtyBits bitMask,
+                            const state::ExtendedDirtyBits extendedBitMask,
+                            const state::DirtyObjects &objectMask,
                             Command command);
-    angle::Result syncDirtyBits(Command command);
-    angle::Result syncDirtyBits(const State::DirtyBits &bitMask, Command command);
-    angle::Result syncDirtyObjects(const State::DirtyObjects &objectMask, Command command);
+    angle::Result syncDirtyBits(const state::DirtyBits bitMask,
+                                const state::ExtendedDirtyBits extendedBitMask,
+                                Command command);
+    angle::Result syncDirtyObjects(const state::DirtyObjects &objectMask, Command command);
     angle::Result syncStateForReadPixels();
     angle::Result syncStateForTexImage();
-    angle::Result syncStateForBlit();
+    angle::Result syncStateForBlit(GLbitfield mask);
     angle::Result syncStateForClear();
     angle::Result syncTextureForCopy(Texture *texture);
 
     VertexArray *checkVertexArrayAllocation(VertexArrayID vertexArrayHandle);
     TransformFeedback *checkTransformFeedbackAllocation(TransformFeedbackID transformFeedback);
 
-    angle::Result onProgramLink(Program *programObject);
-
     void detachBuffer(Buffer *buffer);
     void detachTexture(TextureID texture);
     void detachFramebuffer(FramebufferID framebuffer);
@@ -689,6 +915,7 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     egl::Error unsetDefaultFramebuffer();
 
     void initRendererString();
+    void initVendorString();
     void initVersionStrings();
     void initExtensionStrings();
 
@@ -710,14 +937,17 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
                                             GLsizei height,
                                             MultisamplingMode mode);
 
+    void onUniformBlockBindingUpdated(GLuint uniformBlockIndex);
+
+    void endTilingImplicit();
+
     State mState;
     bool mShared;
-    bool mSkipValidation;
     bool mDisplayTextureShareGroup;
     bool mDisplaySemaphoreShareGroup;
 
     // Recorded errors
-    ErrorSet mErrors;
+    mutable ErrorSet mErrors;
 
     // Stores for each buffer binding type whether is it allowed to be used in this context.
     angle::PackedEnumBitSet<BufferBinding> mValidBufferBindings;
@@ -749,6 +979,7 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     TransformFeedbackMap mTransformFeedbackMap;
     HandleAllocator mTransformFeedbackHandleAllocator;
 
+    const char *mVendorString;
     const char *mVersionString;
     const char *mShadingLanguageString;
     const char *mRendererString;
@@ -762,11 +993,6 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
 
     // Current/lost context flags
     bool mHasBeenCurrent;
-    bool mContextLost;  // Set with setContextLost so that we also set mSkipValidation=false.
-    GraphicsResetStatus mResetStatus;
-    bool mContextLostForced;
-    GLenum mResetStrategy;
-    const bool mRobustAccess;
     const bool mSurfacelessSupported;
     egl::Surface *mCurrentDrawSurface;
     egl::Surface *mCurrentReadSurface;
@@ -775,31 +1001,27 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     bool mBufferAccessValidationEnabled;
     const bool mExtensionsEnabled;
     MemoryProgramCache *mMemoryProgramCache;
+    MemoryShaderCache *mMemoryShaderCache;
 
-    State::DirtyObjects mDrawDirtyObjects;
+    state::DirtyObjects mDrawDirtyObjects;
 
     StateCache mStateCache;
+    PrivateStateCache mPrivateStateCache;
 
-    State::DirtyBits mAllDirtyBits;
-    State::DirtyBits mTexImageDirtyBits;
-    State::DirtyObjects mTexImageDirtyObjects;
-    State::DirtyBits mReadPixelsDirtyBits;
-    State::DirtyObjects mReadPixelsDirtyObjects;
-    State::DirtyBits mClearDirtyBits;
-    State::DirtyObjects mClearDirtyObjects;
-    State::DirtyBits mBlitDirtyBits;
-    State::DirtyObjects mBlitDirtyObjects;
-    State::DirtyBits mComputeDirtyBits;
-    State::DirtyObjects mComputeDirtyObjects;
-    State::DirtyBits mCopyImageDirtyBits;
-    State::DirtyObjects mCopyImageDirtyObjects;
-    State::DirtyBits mReadInvalidateDirtyBits;
-    State::DirtyBits mDrawInvalidateDirtyBits;
+    state::DirtyObjects mTexImageDirtyObjects;
+    state::DirtyObjects mReadPixelsDirtyObjects;
+    state::DirtyObjects mClearDirtyObjects;
+    state::DirtyObjects mBlitDirtyObjects;
+    state::DirtyObjects mComputeDirtyObjects;
+    state::DirtyBits mCopyImageDirtyBits;
+    state::DirtyObjects mCopyImageDirtyObjects;
+    state::DirtyObjects mTilingDirtyObjects;
 
     // Binding to container objects that use dependent state updates.
     angle::ObserverBinding mVertexArrayObserverBinding;
     angle::ObserverBinding mDrawFramebufferObserverBinding;
     angle::ObserverBinding mReadFramebufferObserverBinding;
+    angle::ObserverBinding mProgramObserverBinding;
     angle::ObserverBinding mProgramPipelineObserverBinding;
     std::vector<angle::ObserverBinding> mUniformBufferObserverBindings;
     std::vector<angle::ObserverBinding> mAtomicCounterBufferObserverBindings;
@@ -811,12 +1033,6 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
     mutable Optional<angle::ScratchBuffer> mScratchBuffer;
     mutable Optional<angle::ScratchBuffer> mZeroFilledBuffer;
 
-    // Single-threaded pool may not always be initialized. It currently depends on the extension
-    // GL_KHR_parallel_shader_compile being disabled.
-    std::shared_ptr<angle::WorkerThreadPool> mSingleThreadPool;
-    // Multithreaded pool will always be initialized so it can be used for more generic work.
-    std::shared_ptr<angle::WorkerThreadPool> mMultiThreadPool;
-
     // Note: we use a raw pointer here so we can exclude frame capture sources from the build.
     std::unique_ptr<angle::FrameCapture> mFrameCapture;
 
@@ -827,10 +1043,9 @@ class Context final : public egl::LabeledObject, angle::NonCopyable, public angl
 
     OverlayType mOverlay;
 
-    const bool mIsExternal;
-    const bool mSaveAndRestoreState;
-
     bool mIsDestroyed;
+
+    std::unique_ptr<Framebuffer> mDefaultFramebuffer;
 };
 
 class [[nodiscard]] ScopedContextRef
@@ -856,13 +1071,15 @@ class [[nodiscard]] ScopedContextRef
 };
 
 // Thread-local current valid context bound to the thread.
-#if defined(ANGLE_PLATFORM_APPLE)
+#if defined(ANGLE_PLATFORM_APPLE) || defined(ANGLE_USE_STATIC_THREAD_LOCAL_VARIABLES)
 extern Context *GetCurrentValidContextTLS();
 extern void SetCurrentValidContextTLS(Context *context);
 #else
 extern thread_local Context *gCurrentValidContext;
 #endif
 
+extern void SetCurrentValidContext(Context *context);
+
 }  // namespace gl
 
 #endif  // LIBANGLE_CONTEXT_H_
