@@ -673,7 +673,7 @@ angle::Result VertexArrayVk::convertVertexBufferCPU(ContextVk *contextVk,
     }
 
     uint8_t *src = nullptr;
-    ANGLE_TRY(srcBuffer->mapImpl(contextVk, GL_MAP_READ_BIT, reinterpret_cast<void **>(&src)));
+    ANGLE_TRY(srcBuffer->mapForReadAccessOnly(contextVk, reinterpret_cast<void **>(&src)));
     uint32_t srcStride      = conversion->getCacheKey().stride;
 
     if (conversion->isEntireBufferDirty())
@@ -717,7 +717,7 @@ angle::Result VertexArrayVk::convertVertexBufferCPU(ContextVk *contextVk,
     }
 
     conversion->clearDirty();
-    ANGLE_TRY(srcBuffer->unmapImpl(contextVk));
+    ANGLE_TRY(srcBuffer->unmapReadAccessOnly(contextVk));
 
     return angle::Result::Continue;
 }
@@ -755,18 +755,23 @@ angle::Result VertexArrayVk::syncState(const gl::Context *context,
                 // internal storage and take action if buffer storage has changed while not
                 // observing.
                 if (contextVk->getFeatures().compressVertexData.enabled ||
-                    mContentsObservers->any())
+                    mContentsObserverBindingsMask.any())
                 {
                     // We may have lost buffer content change when it became non-current. In that
                     // case we always assume buffer has changed. If compressVertexData.enabled is
                     // true, it also depends on buffer usage which may have changed.
-                    iter.setLaterBits(
-                        gl::VertexArray::DirtyBits(mState.getBufferBindingMask().to_ulong()
-                                                   << gl::VertexArray::DIRTY_BIT_BINDING_0));
+                    uint64_t bits = mState.getBufferBindingMask().bits();
+                    bits <<= gl::VertexArray::DIRTY_BIT_BINDING_0;
+                    iter.setLaterBits(gl::VertexArray::DirtyBits(bits));
                 }
                 else
                 {
-                    for (size_t bindingIndex : mState.getBufferBindingMask())
+                    // Element buffer is not in bindings yet, has to handle separately.
+                    iter.setLaterBit(gl::VertexArray::DIRTY_BIT_ELEMENT_ARRAY_BUFFER);
+                    gl::VertexArrayBufferBindingMask bindingMask = mState.getBufferBindingMask();
+                    bindingMask.reset(gl::kElementArrayBufferIndex);
+
+                    for (size_t bindingIndex : bindingMask)
                     {
                         const gl::Buffer *bufferGL    = bindings[bindingIndex].getBuffer().get();
                         vk::BufferSerial bufferSerial = vk::GetImpl(bufferGL)->getBufferSerial();
@@ -935,7 +940,7 @@ angle::Result VertexArrayVk::syncDirtyAttrib(ContextVk *contextVk,
 
         if (bufferGL)
         {
-            mContentsObservers->disableForBuffer(bufferGL, static_cast<uint32_t>(attribIndex));
+            mContentsObserverBindingsMask.reset(attrib.bindingIndex);
         }
 
         if (!isStreamingVertexAttrib && bufferGL->getSize() > 0)
@@ -968,7 +973,7 @@ angle::Result VertexArrayVk::syncDirtyAttrib(ContextVk *contextVk,
                 ASSERT(vertexFormat.getVertexInputAlignment(compressed) <=
                        vk::kVertexBufferAlignment);
 
-                mContentsObservers->enableForBuffer(bufferGL, static_cast<uint32_t>(attribIndex));
+                mContentsObserverBindingsMask.set(attrib.bindingIndex);
 
                 WarnOnVertexFormatConversion(contextVk, vertexFormat, compressed, true);
 
@@ -1090,10 +1095,17 @@ angle::Result VertexArrayVk::syncDirtyAttrib(ContextVk *contextVk,
             mCurrentArrayBufferHandles[attribIndex] = emptyBuffer.getBuffer().getHandle();
             mCurrentArrayBufferOffsets[attribIndex] = emptyBuffer.getOffset();
 
-            bool combined = ShouldCombineAttributes(renderer, attrib, binding);
-            mCurrentArrayBufferStrides[attribIndex] =
-                combined ? binding.getStride()
-                         : vertexFormat.getActualBufferFormat(compressed).pixelBytes;
+            if (isStreamingVertexAttrib)
+            {
+                bool combined = ShouldCombineAttributes(renderer, attrib, binding);
+                mCurrentArrayBufferStrides[attribIndex] =
+                    combined ? binding.getStride()
+                             : vertexFormat.getActualBufferFormat(compressed).pixelBytes;
+            }
+            else
+            {
+                mCurrentArrayBufferStrides[attribIndex] = 0;
+            }
         }
 
         if (bufferOnly)
@@ -1290,7 +1302,7 @@ angle::Result VertexArrayVk::updateStreamedAttribs(const gl::Context *context,
                         // Map buffer to expand attribs for divisor emulation
                         BufferVk *bufferVk = vk::GetImpl(binding.getBuffer().get());
                         void *buffSrc      = nullptr;
-                        ANGLE_TRY(bufferVk->mapImpl(contextVk, GL_MAP_READ_BIT, &buffSrc));
+                        ANGLE_TRY(bufferVk->mapForReadAccessOnly(contextVk, &buffSrc));
                         src = reinterpret_cast<const uint8_t *>(buffSrc) + binding.getOffset();
 
                         uint32_t srcAttributeSize =
@@ -1303,7 +1315,7 @@ angle::Result VertexArrayVk::updateStreamedAttribs(const gl::Context *context,
                             stride, vertexFormat.getVertexLoadFunction(compressed), divisor,
                             numVertices));
 
-                        ANGLE_TRY(bufferVk->unmapImpl(contextVk));
+                        ANGLE_TRY(bufferVk->unmapReadAccessOnly(contextVk));
                     }
                     else if (contextVk->getExtensions().robustnessAny())
                     {
@@ -1336,6 +1348,17 @@ angle::Result VertexArrayVk::updateStreamedAttribs(const gl::Context *context,
                                            vertexFormat.getVertexLoadFunction(compressed)));
             }
         }
+        else if (attrib.pointer == nullptr)
+        {
+            // Set them to the initial value.
+            vk::BufferHelper &emptyBuffer            = contextVk->getEmptyBuffer();
+            mCurrentArrayBuffers[attribIndex]        = &emptyBuffer;
+            mCurrentArrayBufferHandles[attribIndex]  = emptyBuffer.getBuffer().getHandle();
+            mCurrentArrayBufferOffsets[attribIndex]  = 0;
+            mCurrentArrayBufferStrides[attribIndex]  = 0;
+            mCurrentArrayBufferDivisors[attribIndex] = 0;
+            continue;
+        }
         else
         {
             ASSERT(binding.getBuffer().get() == nullptr);
