@@ -20,8 +20,7 @@ WindowSurfaceVkHeadless::WindowSurfaceVkHeadless(const egl::SurfaceState &surfac
 
 WindowSurfaceVkHeadless::~WindowSurfaceVkHeadless() {}
 
-angle::Result WindowSurfaceVkHeadless::createSurfaceVk(vk::ErrorContext *context,
-                                                       gl::Extents *extentsOut)
+angle::Result WindowSurfaceVkHeadless::createSurfaceVk(vk::ErrorContext *context)
 {
     vk::Renderer *renderer = context->getRenderer();
     ASSERT(renderer != nullptr);
@@ -32,16 +31,12 @@ angle::Result WindowSurfaceVkHeadless::createSurfaceVk(vk::ErrorContext *context
 
     ANGLE_VK_TRY(context, vkCreateHeadlessSurfaceEXT(instance, &createInfo, nullptr, &mSurface));
 
-    return getCurrentWindowSize(context, extentsOut);
+    return angle::Result::Continue;
 }
 
 angle::Result WindowSurfaceVkHeadless::getCurrentWindowSize(vk::ErrorContext *context,
-                                                            gl::Extents *extentsOut)
+                                                            gl::Extents *extentsOut) const
 {
-    const VkPhysicalDevice &physicalDevice = context->getRenderer()->getPhysicalDevice();
-    ANGLE_VK_TRY(context, vkGetPhysicalDeviceSurfaceCapabilitiesKHR(physicalDevice, mSurface,
-                                                                    &mSurfaceCaps));
-
     // Spec: "For headless surfaces, currentExtent is the reserved value (0xFFFFFFFF, 0xFFFFFFFF).
     // Whatever the application sets a swapchain's imageExtent to will be the size of the surface,
     // after the first image is presented."
@@ -50,12 +45,7 @@ angle::Result WindowSurfaceVkHeadless::getCurrentWindowSize(vk::ErrorContext *co
     angle::vk::SimpleDisplayWindow *simpleWindow =
         reinterpret_cast<angle::vk::SimpleDisplayWindow *>(mNativeWindowType);
 
-    // Update surface extent before output the new extent.
-    mSurfaceCaps.currentExtent.width  = simpleWindow->width;
-    mSurfaceCaps.currentExtent.height = simpleWindow->height;
-
-    *extentsOut =
-        gl::Extents(mSurfaceCaps.currentExtent.width, mSurfaceCaps.currentExtent.height, 1);
+    *extentsOut = gl::Extents(simpleWindow->width, simpleWindow->height, 1);
 
     return angle::Result::Continue;
 }
