@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010 Apple Inc. All rights reserved.
- * Portions Copyright (c) 2010 Motorola Mobility, Inc.  All rights reserved.
+ * Portions Copyright (c) 2010 Motorola Mobility, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -96,13 +96,13 @@ RunLoop::~RunLoop()
 
 void RunLoop::run()
 {
-    RunLoop& runLoop = RunLoop::current();
-    GMainContext* mainContext = runLoop.m_mainContext.get();
+    Ref runLoop = RunLoop::currentSingleton();
+    GMainContext* mainContext = runLoop->m_mainContext.get();
 
     // The innermost main loop should always be there.
-    ASSERT(!runLoop.m_mainLoops.isEmpty());
+    ASSERT(!runLoop->m_mainLoops.isEmpty());
 
-    GMainLoop* innermostLoop = runLoop.m_mainLoops[0].get();
+    GMainLoop* innermostLoop = runLoop->m_mainLoops[0].get();
     if (!g_main_loop_is_running(innermostLoop)) {
         g_main_context_push_thread_default(mainContext);
         g_main_loop_run(innermostLoop);
@@ -112,13 +112,13 @@ void RunLoop::run()
 
     // Create and run a nested loop if the innermost one was already running.
     GMainLoop* nestedMainLoop = g_main_loop_new(mainContext, FALSE);
-    runLoop.m_mainLoops.append(adoptGRef(nestedMainLoop));
+    runLoop->m_mainLoops.append(adoptGRef(nestedMainLoop));
 
     g_main_context_push_thread_default(mainContext);
     g_main_loop_run(nestedMainLoop);
     g_main_context_pop_thread_default(mainContext);
 
-    runLoop.m_mainLoops.removeLast();
+    runLoop->m_mainLoops.removeLast();
 }
 
 void RunLoop::stop()
@@ -149,7 +149,7 @@ void RunLoop::observe(const RunLoop::Observer& observer)
 
 void RunLoop::notify(RunLoop::Event event, const char* name)
 {
-    if (m_observers.computesEmpty())
+    if (m_observers.isEmptyIgnoringNullReferences())
         return;
 
     m_observers.forEach([event, name = String::fromUTF8(name)](auto& observer) {
@@ -157,15 +157,16 @@ void RunLoop::notify(RunLoop::Event event, const char* name)
     });
 }
 
-RunLoop::TimerBase::TimerBase(RunLoop& runLoop)
-    : m_runLoop(runLoop)
+RunLoop::TimerBase::TimerBase(Ref<RunLoop>&& runLoop, ASCIILiteral description)
+    : m_runLoop(WTFMove(runLoop))
+    , m_description(description)
     , m_source(adoptGRef(g_source_new(&RunLoop::s_runLoopSourceFunctions, sizeof(RunLoopSource))))
 {
     auto& runLoopSource = *reinterpret_cast<RunLoopSource*>(m_source.get());
     runLoopSource.runLoop = m_runLoop.ptr();
 
     g_source_set_priority(m_source.get(), RunLoopSourcePriority::RunLoopTimer);
-    g_source_set_name(m_source.get(), "[WebKit] RunLoop::Timer work");
+    g_source_set_name(m_source.get(), m_description);
     g_source_set_callback(m_source.get(), [](gpointer userData) -> gboolean {
         // fired() executes the user's callback. It may destroy timer,
         // so we must check if the source is still active afterwards
@@ -187,11 +188,6 @@ RunLoop::TimerBase::~TimerBase()
     g_source_destroy(m_source.get());
 }
 
-void RunLoop::TimerBase::setName(const char* name)
-{
-    g_source_set_name(m_source.get(), name);
-}
-
 void RunLoop::TimerBase::setPriority(int priority)
 {
     g_source_set_priority(m_source.get(), priority);
