@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -49,7 +49,7 @@ private:
     bool equals(const CSSNumericValue& other) const final { return equalsImpl<CSSMathMax>(other); }
 
     CSSMathMax(Vector<Ref<CSSNumericValue>>&&, CSSNumericType&&);
-    Ref<CSSNumericArray> m_values;
+    const Ref<CSSNumericArray> m_values;
 };
 
 } // namespace WebCore
