@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007, 2014, 2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2007-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,35 +29,43 @@
 #include "ApplicationCacheHost.h"
 #include "BackForwardController.h"
 #include "CachedPage.h"
-#include "DOMWindow.h"
 #include "DeviceMotionController.h"
 #include "DeviceOrientationController.h"
 #include "DiagnosticLoggingClient.h"
 #include "DiagnosticLoggingKeys.h"
+#include "DiagnosticLoggingResultType.h"
 #include "Document.h"
+#include "DocumentInlines.h"
 #include "DocumentLoader.h"
 #include "FocusController.h"
-#include "Frame.h"
+#include "FrameDestructionObserverInlines.h"
 #include "FrameLoader.h"
-#include "FrameLoaderClient.h"
-#include "FrameView.h"
+#include "HTTPParsers.h"
 #include "HistoryController.h"
-#include "IgnoreOpensDuringUnloadCountIncrementer.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
+#include "LocalFrameView.h"
 #include "Logging.h"
 #include "Page.h"
 #include "Quirks.h"
 #include "ScriptDisallowedScope.h"
+#include "SecurityOriginHash.h"
 #include "Settings.h"
 #include "SubframeLoader.h"
+#include "UnloadCountIncrementer.h"
 #include <pal/Logging.h>
 #include <wtf/MemoryPressureHandler.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/SetForScope.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/text/CString.h>
-#include <wtf/text/StringConcatenate.h>
+#include <wtf/text/MakeString.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(BackForwardCache);
+
 #define PCLOG(...) LOG(BackForwardCache, "%*s%s", indentLevel*4, "", makeString(__VA_ARGS__).utf8().data())
 
 static inline void logBackForwardCacheFailureDiagnosticMessage(DiagnosticLoggingClient& client, const String& reason)
@@ -73,95 +81,96 @@ static inline void logBackForwardCacheFailureDiagnosticMessage(Page* page, const
     logBackForwardCacheFailureDiagnosticMessage(page->diagnosticLoggingClient(), reason);
 }
 
-static bool canCacheFrame(Frame& frame, DiagnosticLoggingClient& diagnosticLoggingClient, unsigned indentLevel)
+static bool canCacheFrame(LocalFrame& frame, DiagnosticLoggingClient& diagnosticLoggingClient, unsigned indentLevel)
 {
-    PCLOG("+---");
-    FrameLoader& frameLoader = frame.loader();
+    PCLOG("+---"_s);
+    Ref frameLoader = frame.loader();
 
     // Prevent page caching if a subframe is still in provisional load stage.
     // We only do this check for subframes because the main frame is reused when navigating to a new page.
-    if (!frame.isMainFrame() && frameLoader.state() == FrameState::Provisional) {
-        PCLOG("   -Frame is in provisional load stage");
+    if (!frame.isMainFrame() && frameLoader->state() == FrameState::Provisional) {
+        PCLOG("   -Frame is in provisional load stage"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::provisionalLoadKey());
         return false;
     }
 
-    if (frame.isMainFrame() && frameLoader.stateMachine().isDisplayingInitialEmptyDocument()) {
-        PCLOG("   -MainFrame is displaying initial empty document");
+    if (frame.isMainFrame() && frameLoader->stateMachine().isDisplayingInitialEmptyDocument()) {
+        PCLOG("   -MainFrame is displaying initial empty document"_s);
         return false;
     }
 
-    if (!frame.document()) {
-        PCLOG("   -Frame has no document");
+    RefPtr document = frame.document();
+    if (!document) {
+        PCLOG("   -Frame has no document"_s);
         return false;
     }
 
-    if (frame.document()->shouldPreventEnteringBackForwardCacheForTesting()) {
-        PCLOG("   -Back/Forward caching is disabled for testing");
+    if (document->shouldPreventEnteringBackForwardCacheForTesting()) {
+        PCLOG("   -Back/Forward caching is disabled for testing"_s);
         return false;
     }
 
-    if (!frame.document()->frame()) {
-        PCLOG("   -Document is detached from frame");
+    if (!document->frame()) {
+        PCLOG("   -Document is detached from frame"_s);
         ASSERT_NOT_REACHED();
         return false;
     }
 
-    DocumentLoader* documentLoader = frameLoader.documentLoader();
+    RefPtr documentLoader = frameLoader->documentLoader();
     if (!documentLoader) {
-        PCLOG("   -There is no DocumentLoader object");
+        PCLOG("   -There is no DocumentLoader object"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::noDocumentLoaderKey());
         return false;
     }
 
     URL currentURL = documentLoader->url();
-    URL newURL = frameLoader.provisionalDocumentLoader() ? frameLoader.provisionalDocumentLoader()->url() : URL();
+    URL newURL = frameLoader->provisionalDocumentLoader() ? frameLoader->provisionalDocumentLoader()->url() : URL();
     if (!newURL.isEmpty())
-        PCLOG(" Determining if frame can be cached navigating from (", currentURL.string(), ") to (", newURL.string(), "):");
+        PCLOG(" Determining if frame can be cached navigating from ("_s, currentURL.string(), ") to ("_s, newURL.string(), "):"_s);
     else
-        PCLOG(" Determining if subframe with URL (", currentURL.string(), ") can be cached:");
+        PCLOG(" Determining if subframe with URL ("_s, currentURL.string(), ") can be cached:"_s);
 
     bool isCacheable = true;
 
-    if (frame.isMainFrame() && frame.document()->quirks().shouldBypassBackForwardCache()) {
-        PCLOG("   -Disabled by site-specific quirk");
+    if (frame.isMainFrame() && document->quirks().shouldBypassBackForwardCache()) {
+        PCLOG("   -Disabled by site-specific quirk"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::siteSpecificQuirkKey());
         isCacheable = false;
     }
 
     // Do not cache error pages (these can be recognized as pages with substitute data or unreachable URLs).
     if (documentLoader->substituteData().isValid() && !documentLoader->substituteData().failingURL().isEmpty()) {
-        PCLOG("   -Frame is an error page");
+        PCLOG("   -Frame is an error page"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::isErrorPageKey());
         isCacheable = false;
     }
-    if (frame.isMainFrame() && frame.document() && frame.document()->url().protocolIs("https"_s) && documentLoader->response().cacheControlContainsNoStore()) {
-        PCLOG("   -Frame is HTTPS, and cache control prohibits storing");
+    if (frame.isMainFrame() && document && document->url().protocolIs("https"_s) && documentLoader->response().cacheControlContainsNoStore()) {
+        PCLOG("   -Frame is HTTPS, and cache control prohibits storing"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::httpsNoStoreKey());
         isCacheable = false;
     }
-    if (frame.isMainFrame() && !frameLoader.history().currentItem()) {
-        PCLOG("   -Main frame has no current history item");
+    if (frame.isMainFrame() && !frame.loader().history().currentItem()) {
+        PCLOG("   -Main frame has no current history item"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::noCurrentHistoryItemKey());
         isCacheable = false;
     }
     if (frame.isMainFrame() && frame.view() && !frame.view()->isVisuallyNonEmpty()) {
-        PCLOG("   -Main frame is visually empty");
+        PCLOG("   -Main frame is visually empty"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::visuallyEmptyKey());
         isCacheable = false;
     }
-    if (frameLoader.quickRedirectComing()) {
-        PCLOG("   -Quick redirect is coming");
+    if (frameLoader->quickRedirectComing()) {
+        PCLOG("   -Quick redirect is coming"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::quirkRedirectComingKey());
         isCacheable = false;
     }
     if (documentLoader->isLoading()) {
-        PCLOG("   -DocumentLoader is still loading");
+        PCLOG("   -DocumentLoader is still loading"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::isLoadingKey());
         isCacheable = false;
     }
     if (documentLoader->isStopping()) {
-        PCLOG("   -DocumentLoader is in the middle of stopping");
+        PCLOG("   -DocumentLoader is in the middle of stopping"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::documentLoaderStoppingKey());
         isCacheable = false;
     }
@@ -169,23 +178,26 @@ static bool canCacheFrame(Frame& frame, DiagnosticLoggingClient& diagnosticLoggi
     // FIXME: We should investigating caching frames that have an associated
     // application cache. <rdar://problem/5917899> tracks that work.
     if (!documentLoader->applicationCacheHost().canCacheInBackForwardCache()) {
-        PCLOG("   -The DocumentLoader uses an application cache");
+        PCLOG("   -The DocumentLoader uses an application cache"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::applicationCacheKey());
         isCacheable = false;
     }
-    if (!frameLoader.client().canCachePage()) {
-        PCLOG("   -The client says this frame cannot be cached");
+    if (!frameLoader->client().canCachePage()) {
+        PCLOG("   -The client says this frame cannot be cached"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::deniedByClientKey());
         isCacheable = false;
     }
 
-    for (Frame* child = frame.tree().firstChild(); child; child = child->tree().nextSibling()) {
-        if (!canCacheFrame(*child, diagnosticLoggingClient, indentLevel + 1))
+    for (RefPtr child = frame.tree().firstChild(); child; child = child->tree().nextSibling()) {
+        auto* localChild = dynamicDowncast<LocalFrame>(child.get());
+        if (!localChild)
+            continue;
+        if (!canCacheFrame(*localChild, diagnosticLoggingClient, indentLevel + 1))
             isCacheable = false;
     }
     
-    PCLOG(isCacheable ? " Frame CAN be cached" : " Frame CANNOT be cached");
-    PCLOG("+---");
+    PCLOG(isCacheable ? " Frame CAN be cached"_s : " Frame CANNOT be cached"_s);
+    PCLOG("+---"_s);
     
     return isCacheable;
 }
@@ -195,65 +207,68 @@ static bool canCachePage(Page& page)
     RELEASE_ASSERT(!page.isRestoringCachedPage());
 
     unsigned indentLevel = 0;
-    PCLOG("--------\n Determining if page can be cached:");
+    PCLOG("--------\n Determining if page can be cached:"_s);
 
-    DiagnosticLoggingClient& diagnosticLoggingClient = page.diagnosticLoggingClient();
-    bool isCacheable = canCacheFrame(page.mainFrame(), diagnosticLoggingClient, indentLevel + 1);
+    CheckedRef diagnosticLoggingClient = page.diagnosticLoggingClient();
+    RefPtr localMainFrame = page.localMainFrame();
+    if (!localMainFrame)
+        return false;
+    bool isCacheable = canCacheFrame(*localMainFrame, diagnosticLoggingClient, indentLevel + 1);
 
-    if (!page.settings().usesBackForwardCache() || page.isResourceCachingDisabledByWebInspector()) {
-        PCLOG("   -Page settings says b/f cache disabled");
+    if (!page.settings().usesBackForwardCache() || page.isResourceCachingDisabledByWebInspector() || page.settings().siteIsolationEnabled()) {
+        PCLOG("   -Page settings says b/f cache disabled"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::isDisabledKey());
         isCacheable = false;
     }
 #if ENABLE(DEVICE_ORIENTATION) && !PLATFORM(IOS_FAMILY)
     if (DeviceMotionController::isActiveAt(&page)) {
-        PCLOG("   -Page is using DeviceMotion");
+        PCLOG("   -Page is using DeviceMotion"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::deviceMotionKey());
         isCacheable = false;
     }
     if (DeviceOrientationController::isActiveAt(&page)) {
-        PCLOG("   -Page is using DeviceOrientation");
+        PCLOG("   -Page is using DeviceOrientation"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::deviceOrientationKey());
         isCacheable = false;
     }
 #endif
 
-    FrameLoadType loadType = page.mainFrame().loader().loadType();
+    FrameLoadType loadType = localMainFrame->loader().loadType();
     switch (loadType) {
     case FrameLoadType::Reload:
         // No point writing to the cache on a reload, since we will just write over it again when we leave that page.
-        PCLOG("   -Load type is: Reload");
+        PCLOG("   -Load type is: Reload"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::reloadKey());
         isCacheable = false;
         break;
     case FrameLoadType::Same: // user loads same URL again (but not reload button)
         // No point writing to the cache on a same load, since we will just write over it again when we leave that page.
-        PCLOG("   -Load type is: Same");
+        PCLOG("   -Load type is: Same"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::sameLoadKey());
         isCacheable = false;
         break;
     case FrameLoadType::RedirectWithLockedBackForwardList:
         // Don't write to the cache if in the middle of a redirect, since we will want to store the final page we end up on.
-        PCLOG("   -Load type is: RedirectWithLockedBackForwardList");
+        PCLOG("   -Load type is: RedirectWithLockedBackForwardList"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::redirectKey());
         isCacheable = false;
         break;
     case FrameLoadType::Replace:
         // No point writing to the cache on a replace, since we will just write over it again when we leave that page.
-        PCLOG("   -Load type is: Replace");
+        PCLOG("   -Load type is: Replace"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::replaceKey());
         isCacheable = false;
         break;
     case FrameLoadType::ReloadFromOrigin: {
         // No point writing to the cache on a reload, since we will just write over it again when we leave that page.
-        PCLOG("   -Load type is: ReloadFromOrigin");
+        PCLOG("   -Load type is: ReloadFromOrigin"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::reloadFromOriginKey());
         isCacheable = false;
         break;
     }
     case FrameLoadType::ReloadExpiredOnly: {
         // No point writing to the cache on a reload, since we will just write over it again when we leave that page.
-        PCLOG("   -Load type is: ReloadRevalidatingExpired");
+        PCLOG("   -Load type is: ReloadRevalidatingExpired"_s);
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::reloadRevalidatingExpiredKey());
         isCacheable = false;
         break;
@@ -265,13 +280,26 @@ static bool canCachePage(Page& page)
         // Cacheable.
         break;
     }
+
+    // If this is a same-origin navigation and the navigated-to main resource serves the
+    // `Clear-Site-Data: "cache"` HTTP header, then we shouldn't cache the current page.
+    if (RefPtr provisionalDocumentLoader = localMainFrame->loader().provisionalDocumentLoader()) {
+        if (provisionalDocumentLoader->responseClearSiteDataValues().contains(ClearSiteDataValue::Cache)) {
+            if (RefPtr topDocument = localMainFrame->document()) {
+                if (topDocument->protectedSecurityOrigin()->isSameOriginAs(SecurityOrigin::create(provisionalDocumentLoader->response().url()))) {
+                    PCLOG("   -`Clear-Site-Data: cache` HTTP header is present"_s);
+                    isCacheable = false;
+                }
+            }
+        }
+    }
     
     if (isCacheable)
-        PCLOG(" Page CAN be cached\n--------");
+        PCLOG(" Page CAN be cached\n--------"_s);
     else
-        PCLOG(" Page CANNOT be cached\n--------");
+        PCLOG(" Page CANNOT be cached\n--------"_s);
 
-    diagnosticLoggingClient.logDiagnosticMessageWithResult(DiagnosticLoggingKeys::backForwardCacheKey(), DiagnosticLoggingKeys::canCacheKey(), isCacheable ? DiagnosticLoggingResultPass : DiagnosticLoggingResultFail, ShouldSample::No);
+    diagnosticLoggingClient->logDiagnosticMessageWithResult(DiagnosticLoggingKeys::backForwardCacheKey(), DiagnosticLoggingKeys::canCacheKey(), isCacheable ? DiagnosticLoggingResultPass : DiagnosticLoggingResultFail, ShouldSample::No);
     return isCacheable;
 }
 
@@ -294,9 +322,9 @@ BackForwardCache::BackForwardCache()
 void BackForwardCache::dump() const
 {
     WTFLogAlways("Back/Forward Cache:");
-    for (auto& item : m_items) {
-        CachedPage& cachedPage = *item->m_cachedPage;
-        WTFLogAlways("  Page %p, document %p %s", &cachedPage.page(), cachedPage.document(), cachedPage.document() ? cachedPage.document()->url().string().utf8().data() : "");
+    for (auto& item : m_cachedPageMap) {
+        if (auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&item.value))
+            WTFLogAlways("  Page %p, document %p %s", &(*cachedPage)->page(), (*cachedPage)->document(), (*cachedPage)->document() ? (*cachedPage)->document()->url().string().utf8().data() : "");
     }
 }
 
@@ -330,9 +358,11 @@ void BackForwardCache::setMaxSize(unsigned maxSize)
 unsigned BackForwardCache::frameCount() const
 {
     unsigned frameCount = m_items.size();
-    for (auto& item : m_items) {
-        ASSERT(item->m_cachedPage);
-        frameCount += item->m_cachedPage->cachedMainFrame()->descendantFrameCount();
+    for (auto& item : m_cachedPageMap) {
+        if (auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&item.value))
+            frameCount += (*cachedPage)->cachedMainFrame()->descendantFrameCount();
+        else
+            ASSERT(!m_items.contains(item.key));
     }
     
     return frameCount;
@@ -340,28 +370,40 @@ unsigned BackForwardCache::frameCount() const
 
 void BackForwardCache::markPagesForDeviceOrPageScaleChanged(Page& page)
 {
-    for (auto& item : m_items) {
-        CachedPage& cachedPage = *item->m_cachedPage;
-        if (&page.mainFrame() == &cachedPage.cachedMainFrame()->view()->frame())
-            cachedPage.markForDeviceOrPageScaleChanged();
+    for (auto& item : m_cachedPageMap) {
+        auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&item.value);
+        if (!cachedPage) {
+            ASSERT(!m_items.contains(item.key));
+            continue;
+        }
+        if (&page.mainFrame() == &(*cachedPage)->cachedMainFrame()->view()->frame())
+            (*cachedPage)->markForDeviceOrPageScaleChanged();
     }
 }
 
 void BackForwardCache::markPagesForContentsSizeChanged(Page& page)
 {
-    for (auto& item : m_items) {
-        CachedPage& cachedPage = *item->m_cachedPage;
-        if (&page.mainFrame() == &cachedPage.cachedMainFrame()->view()->frame())
-            cachedPage.markForContentsSizeChanged();
+    for (auto& item : m_cachedPageMap) {
+        auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&item.value);
+        if (!cachedPage) {
+            ASSERT(!m_items.contains(item.key));
+            continue;
+        }
+        if (&page.mainFrame() == &(*cachedPage)->cachedMainFrame()->view()->frame())
+            (*cachedPage)->markForContentsSizeChanged();
     }
 }
 
 #if ENABLE(VIDEO)
 void BackForwardCache::markPagesForCaptionPreferencesChanged()
 {
-    for (auto& item : m_items) {
-        ASSERT(item->m_cachedPage);
-        item->m_cachedPage->markForCaptionPreferencesChanged();
+    for (auto& item : m_cachedPageMap) {
+        auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&item.value);
+        if (!cachedPage)  {
+            ASSERT(!m_items.contains(item.key));
+            continue;
+        }
+        CheckedRef { (*cachedPage).get() }->markForCaptionPreferencesChanged();
     }
 }
 #endif
@@ -389,41 +431,30 @@ static void setBackForwardCacheState(Page& page, Document::BackForwardCacheState
     });
 }
 
-// When entering back/forward cache, tear down the render tree before setting the in-cache flag.
-// This maintains the invariant that render trees are never present in the back/forward cache.
-// Note that destruction happens bottom-up so that the main frame's tree dies last.
-static void destroyRenderTree(Frame& mainFrame)
-{
-    for (Frame* frame = mainFrame.tree().traversePrevious(CanWrap::Yes); frame; frame = frame->tree().traversePrevious(CanWrap::No)) {
-        if (!frame->document())
-            continue;
-        auto& document = *frame->document();
-        if (document.hasLivingRenderTree())
-            document.destroyRenderTree();
-    }
-}
-
 static void firePageHideEventRecursively(Frame& frame)
 {
-    auto* document = frame.document();
-    if (!document)
-        return;
+    std::optional<UnloadCountIncrementer> unloadCountIncrementer;
+    if (RefPtr localFrame = dynamicDowncast<LocalFrame>(frame)) {
+        RefPtr document = localFrame->document();
+        if (!document)
+            return;
 
-    // stopLoading() will fire the pagehide event in each subframe and the HTML specification states
-    // that the parent document's ignore-opens-during-unload counter should be incremented while the
-    // pagehide event is being fired in its subframes:
-    // https://html.spec.whatwg.org/multipage/browsers.html#unload-a-document
-    IgnoreOpensDuringUnloadCountIncrementer ignoreOpensDuringUnloadCountIncrementer(document);
+        // stopLoading() will fire the pagehide event in each subframe and the HTML specification states
+        // that the parent document's ignore-opens-during-unload counter should be incremented while the
+        // pagehide event is being fired in its subframes:
+        // https://html.spec.whatwg.org/multipage/browsers.html#unload-a-document
+        unloadCountIncrementer.emplace(document.get());
 
-    frame.loader().stopLoading(UnloadEventPolicy::UnloadAndPageHide);
-
-    for (RefPtr<Frame> child = frame.tree().firstChild(); child; child = child->tree().nextSibling())
+        localFrame->loader().stopLoading(UnloadEventPolicy::UnloadAndPageHide);
+    }
+    for (RefPtr child = frame.tree().firstChild(); child; child = child->tree().nextSibling())
         firePageHideEventRecursively(*child);
 }
 
 std::unique_ptr<CachedPage> BackForwardCache::trySuspendPage(Page& page, ForceSuspension forceSuspension)
 {
-    page.mainFrame().loader().stopForBackForwardCache();
+    Ref mainFrame = page.mainFrame();
+    mainFrame->stopForBackForwardCache();
 
     if (forceSuspension == ForceSuspension::No && !canCache(page))
         return nullptr;
@@ -434,17 +465,17 @@ std::unique_ptr<CachedPage> BackForwardCache::trySuspendPage(Page& page, ForceSu
 
     // Focus the main frame, defocusing a focused subframe (if we have one). We do this here,
     // before the page enters the back/forward cache, while we still can dispatch DOM blur/focus events.
-    if (CheckedRef focusController { page.focusController() }; focusController->focusedFrame())
-        focusController->setFocusedFrame(&page.mainFrame());
+    if (CheckedRef focusController = page.focusController(); focusController->focusedLocalFrame())
+        focusController->setFocusedFrame(mainFrame.ptr());
 
     // Fire the pagehide event in all frames.
-    firePageHideEventRecursively(page.mainFrame());
+    firePageHideEventRecursively(mainFrame);
 
-    destroyRenderTree(page.mainFrame());
+    page.destroyRenderTrees();
 
     // Stop all loads again before checking if we can still cache the page after firing the pagehide
     // event, since the page may have started ping loads in its pagehide event handler.
-    page.mainFrame().loader().stopForBackForwardCache();
+    mainFrame->stopForBackForwardCache();
 
     // Check that the page is still page-cacheable after firing the pagehide event. The JS event handlers
     // could have altered the page in a way that could prevent caching.
@@ -476,13 +507,13 @@ bool BackForwardCache::addIfCacheable(HistoryItem& item, Page* page)
         // Make sure we don't fire any JS events in this scope.
         ScriptDisallowedScope::InMainThread scriptDisallowedScope;
 
-        item.setCachedPage(WTFMove(cachedPage));
-        item.m_pruningReason = PruningReason::None;
-        m_items.add(&item);
+        m_cachedPageMap.set(item.itemID(), makeUniqueRefFromNonNullUniquePtr(WTFMove(cachedPage)));
+        m_items.add(item.itemID());
+        item.notifyChanged();
     }
     prune(PruningReason::ReachedMaxSize);
 
-    RELEASE_LOG(BackForwardCache, "BackForwardCache::addIfCacheable item: %s, size: %u / %u", item.identifier().string().utf8().data(), pageCount(), maxSize());
+    RELEASE_LOG(BackForwardCache, "BackForwardCache::addIfCacheable item: %s, size: %u / %u", item.itemID().toString().utf8().data(), pageCount(), maxSize());
 
     return true;
 }
@@ -496,16 +527,21 @@ std::unique_ptr<CachedPage> BackForwardCache::suspendPage(Page& page)
 
 std::unique_ptr<CachedPage> BackForwardCache::take(HistoryItem& item, Page* page)
 {
-    if (!item.m_cachedPage) {
-        if (item.m_pruningReason != PruningReason::None)
-            logBackForwardCacheFailureDiagnosticMessage(page, pruningReasonToDiagnosticLoggingKey(item.m_pruningReason));
+    auto it = m_cachedPageMap.find(item.itemID());
+    if (it == m_cachedPageMap.end())
+        return nullptr;
+    if (auto* pruningReason = std::get_if<PruningReason>(&it->value)) {
+        ASSERT(!m_items.contains(it->key));
+        if (*pruningReason != PruningReason::None)
+            logBackForwardCacheFailureDiagnosticMessage(page, pruningReasonToDiagnosticLoggingKey(*pruningReason));
         return nullptr;
     }
 
-    m_items.remove(&item);
-    std::unique_ptr<CachedPage> cachedPage = item.takeCachedPage();
+    m_items.remove(item.itemID());
+    auto cachedPage = std::get<UniqueRef<CachedPage>>(m_cachedPageMap.take(it));
+    item.notifyChanged();
 
-    RELEASE_LOG(BackForwardCache, "BackForwardCache::take item: %s, size: %u / %u", item.identifier().string().utf8().data(), pageCount(), maxSize());
+    RELEASE_LOG(BackForwardCache, "BackForwardCache::take item: %s, size: %u / %u", item.itemID().toString().utf8().data(), pageCount(), maxSize());
 
     if (cachedPage->hasExpired() || (page && page->isResourceCachingDisabledByWebInspector())) {
         LOG(BackForwardCache, "Not restoring page for %s from back/forward cache because cache entry has expired", item.url().string().ascii().data());
@@ -513,7 +549,7 @@ std::unique_ptr<CachedPage> BackForwardCache::take(HistoryItem& item, Page* page
         return nullptr;
     }
 
-    return cachedPage;
+    return cachedPage.moveToUniquePtr();
 }
 
 void BackForwardCache::removeAllItemsForPage(Page& page)
@@ -523,57 +559,100 @@ void BackForwardCache::removeAllItemsForPage(Page& page)
     SetForScope inRemoveAllItemsForPageScope { m_isInRemoveAllItemsForPage, true };
 #endif
 
-    for (auto it = m_items.begin(); it != m_items.end();) {
-        // Increment iterator first so it stays valid after the removal.
-        auto current = it;
-        ++it;
-        if (&(*current)->m_cachedPage->page() == &page) {
-            RELEASE_LOG(BackForwardCache, "BackForwardCache::removeAllItemsForPage removing item: %s, size: %u / %u", (*current)->identifier().string().utf8().data(), pageCount() - 1, maxSize());
-            (*current)->setCachedPage(nullptr);
-            m_items.remove(current);
+    m_cachedPageMap.removeIf([&](auto& pair) -> bool {
+        if (auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&pair.value)) {
+            if (&(*cachedPage)->page() == &page) {
+                RELEASE_LOG(BackForwardCache,  "BackForwardCache::removeAllItemsForPage removing item: %s, size: %u / %u", pair.key.toString().utf8().data(), pageCount() - 1, maxSize());
+                m_items.remove(pair.key);
+                return true;
+            }
         }
-    }
+        return false;
+    });
 }
 
 CachedPage* BackForwardCache::get(HistoryItem& item, Page* page)
 {
-    CachedPage* cachedPage = item.m_cachedPage.get();
-    if (!cachedPage) {
-        if (item.m_pruningReason != PruningReason::None)
-            logBackForwardCacheFailureDiagnosticMessage(page, pruningReasonToDiagnosticLoggingKey(item.m_pruningReason));
+    auto it = m_cachedPageMap.find(item.itemID());
+    if (it == m_cachedPageMap.end())
         return nullptr;
-    }
 
-    if (cachedPage->hasExpired() || (page && page->isResourceCachingDisabledByWebInspector())) {
-        LOG(BackForwardCache, "Not restoring page for %s from back/forward cache because cache entry has expired", item.url().string().ascii().data());
-        logBackForwardCacheFailureDiagnosticMessage(page, DiagnosticLoggingKeys::expiredKey());
-        remove(item);
+    return switchOn(it->value, [&](const PruningReason& pruningReason) -> CachedPage* {
+        ASSERT(!m_items.contains(it->key));
+        if (pruningReason != PruningReason::None)
+            logBackForwardCacheFailureDiagnosticMessage(page, pruningReasonToDiagnosticLoggingKey(pruningReason));
         return nullptr;
-    }
-    return cachedPage;
+    }, [&](UniqueRef<CachedPage>& cachedPage) -> CachedPage* {
+        if (cachedPage->hasExpired() || (page && page->isResourceCachingDisabledByWebInspector())) {
+            LOG(BackForwardCache, "Not restoring page for %s from back/forward cache because cache entry has expired", item.url().string().ascii().data());
+            logBackForwardCacheFailureDiagnosticMessage(page, DiagnosticLoggingKeys::expiredKey());
+            remove(item);
+            return nullptr;
+        }
+        return cachedPage.ptr();
+    });
+}
+
+void BackForwardCache::remove(BackForwardItemIdentifier itemID)
+{
+    // Safely ignore attempts to remove items not in the cache.
+    auto it = m_cachedPageMap.find(itemID);
+    if (it == m_cachedPageMap.end() || std::holds_alternative<PruningReason>(it->value))
+        return;
+
+    m_items.remove(itemID);
+    m_cachedPageMap.remove(it);
+
+    RELEASE_LOG(BackForwardCache, "BackForwardCache::remove item: %s, size: %u / %u", itemID.toString().utf8().data(), pageCount(), maxSize());
 }
 
 void BackForwardCache::remove(HistoryItem& item)
 {
-    // Safely ignore attempts to remove items not in the cache.
-    if (!item.m_cachedPage)
-        return;
-
-    m_items.remove(&item);
-    item.setCachedPage(nullptr);
-
-    RELEASE_LOG(BackForwardCache, "BackForwardCache::remove item: %s, size: %u / %u", item.identifier().string().utf8().data(), pageCount(), maxSize());
-
+    remove(item.itemID());
+    item.notifyChanged();
 }
 
 void BackForwardCache::prune(PruningReason pruningReason)
 {
     while (pageCount() > maxSize()) {
         auto oldestItem = m_items.takeFirst();
-        oldestItem->setCachedPage(nullptr);
-        oldestItem->m_pruningReason = pruningReason;
-        RELEASE_LOG(BackForwardCache, "BackForwardCache::prune removing item: %s, size: %u / %u", oldestItem->identifier().string().utf8().data(), pageCount(), maxSize());
+
+        // Take the CachedPage before calling set() so ~CachedPage doesn’t find itself in m_cachedPageMap.
+        auto cachedPage = m_cachedPageMap.take(oldestItem);
+        m_cachedPageMap.set(oldestItem, pruningReason);
+        RELEASE_LOG(BackForwardCache, "BackForwardCache::prune removing item: %s, size: %u / %u", oldestItem.toString().utf8().data(), pageCount(), maxSize());
     }
 }
 
+void BackForwardCache::clearEntriesForOrigins(const HashSet<RefPtr<SecurityOrigin>>& origins)
+{
+    m_cachedPageMap.removeIf([&](auto& pair) -> bool {
+        if (auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&pair.value)) {
+            if (origins.contains(SecurityOrigin::create((*cachedPage)->page().mainFrameURL()))) {
+                RELEASE_LOG(BackForwardCache, "BackForwardCache::clearEntriesForOrigins removing item: %s, size: %u / %u", pair.key.toString().utf8().data(), pageCount() - 1, maxSize());
+                m_items.remove(pair.key);
+                return true;
+            }
+        }
+        return false;
+    });
+}
+
+bool BackForwardCache::isInBackForwardCache(BackForwardItemIdentifier identifier) const
+{
+    auto it = m_cachedPageMap.find(identifier);
+    return it != m_cachedPageMap.end() && std::holds_alternative<UniqueRef<CachedPage>>(it->value);
+}
+
+bool BackForwardCache::hasCachedPageExpired(BackForwardItemIdentifier identifier) const
+{
+    auto it = m_cachedPageMap.find(identifier);
+    if (it == m_cachedPageMap.end())
+        return false;
+    auto* cachedPage = std::get_if<UniqueRef<CachedPage>>(&it->value);
+    if (!cachedPage)
+        return false;
+    return (*cachedPage)->hasExpired();
+}
+
 } // namespace WebCore
