@@ -31,18 +31,24 @@
 #pragma once
 
 #include "BaseButtonInputType.h"
+#include <wtf/TZoneMalloc.h>
 
 namespace WebCore {
 
 class ButtonInputType final : public BaseButtonInputType {
-    template<typename DowncastedType> friend bool isInvalidInputType(const InputType&, const String&);
+    WTF_MAKE_TZONE_ALLOCATED(ButtonInputType);
 public:
+    static Ref<ButtonInputType> create(HTMLInputElement& element)
+    {
+        return adoptRef(*new ButtonInputType(element));
+    }
+
+private:
     explicit ButtonInputType(HTMLInputElement& element)
         : BaseButtonInputType(Type::Button, element)
     {
     }
 
-private:
     const AtomString& formControlType() const final;
 };
 
