@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2025 Apple Inc. All rights reserved.
  * Copyright (C) 2008, 2010 Nokia Corporation and/or its subsidiary(-ies)
  * Copyright (C) 2012, Samsung Electronics. All rights reserved.
  *
@@ -25,13 +25,24 @@
 #include "AppHighlight.h"
 #include "BarcodeDetectorInterface.h"
 #include "ChromeClient.h"
+#include "ColorChooser.h"
 #include "ContactInfo.h"
 #include "ContactsRequestData.h"
+#include "ContainerNodeInlines.h"
+#include "DataListSuggestionPicker.h"
+#include "DateTimeChooser.h"
+#if HAVE(DIGITAL_CREDENTIALS_UI)
+#include "DigitalCredentialsRequestData.h"
+#include "DigitalCredentialsResponseData.h"
+#endif
 #include "Document.h"
+#include "DocumentInlines.h"
 #include "DocumentType.h"
+#include "ExceptionData.h"
 #include "FaceDetectorInterface.h"
 #include "FileList.h"
 #include "FloatRect.h"
+#include "FrameLoader.h"
 #include "FrameTree.h"
 #include "Geolocation.h"
 #include "HTMLFormElement.h"
@@ -45,8 +56,10 @@
 #include "LocalFrameLoaderClient.h"
 #include "Page.h"
 #include "PageGroupLoadDeferrer.h"
+#include "PopupMenu.h"
 #include "PopupOpeningObserver.h"
 #include "RenderObject.h"
+#include "SearchPopupMenu.h"
 #include "Settings.h"
 #include "ShareData.h"
 #include "StorageNamespace.h"
@@ -57,22 +70,13 @@
 #include "WorkerClient.h"
 #include <JavaScriptCore/VM.h>
 #include <wtf/SetForScope.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/Vector.h>
 
-#if ENABLE(INPUT_TYPE_COLOR)
-#include "ColorChooser.h"
-#endif
-
-#if ENABLE(DATALIST_ELEMENT)
-#include "DataListSuggestionPicker.h"
-#endif
-
-#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
-#include "DateTimeChooser.h"
-#endif
-
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Chrome);
+
 using namespace HTMLNames;
 
 Chrome::Chrome(Page& page, UniqueRef<ChromeClient>&& client)
@@ -86,6 +90,11 @@ Chrome::~Chrome()
     m_client->chromeDestroyed();
 }
 
+Ref<Page> Chrome::protectedPage() const
+{
+    return m_page.get();
+}
+
 void Chrome::invalidateRootView(const IntRect& updateRect)
 {
     m_client->invalidateRootView(updateRect);
@@ -104,7 +113,7 @@ void Chrome::invalidateContentsForSlowScroll(const IntRect& updateRect)
 void Chrome::scroll(const IntSize& scrollDelta, const IntRect& rectToScroll, const IntRect& clipRect)
 {
     m_client->scroll(scrollDelta, rectToScroll, clipRect);
-    InspectorInstrumentation::didScroll(m_page);
+    InspectorInstrumentation::didScroll(protectedPage());
 }
 
 IntPoint Chrome::screenToRootView(const IntPoint& point) const
@@ -112,6 +121,11 @@ IntPoint Chrome::screenToRootView(const IntPoint& point) const
     return m_client->screenToRootView(point);
 }
 
+IntPoint Chrome::rootViewToScreen(const IntPoint& point) const
+{
+    return m_client->rootViewToScreen(point);
+}
+
 IntRect Chrome::rootViewToScreen(const IntRect& rect) const
 {
     return m_client->rootViewToScreen(rect);
@@ -127,6 +141,13 @@ IntRect Chrome::rootViewToAccessibilityScreen(const IntRect& rect) const
     return m_client->rootViewToAccessibilityScreen(rect);
 }
 
+#if PLATFORM(IOS_FAMILY)
+void Chrome::relayAccessibilityNotification(String&& notificationName, RetainPtr<NSData>&& notificationData) const
+{
+    return m_client->relayAccessibilityNotification(WTFMove(notificationName), WTFMove(notificationData));
+}
+#endif
+
 PlatformPageClient Chrome::platformPageClient() const
 {
     return m_client->platformPageClient();
@@ -187,20 +208,17 @@ void Chrome::focusedElementChanged(Element* element)
     m_client->focusedElementChanged(element);
 }
 
-void Chrome::focusedFrameChanged(LocalFrame* frame)
+void Chrome::focusedFrameChanged(Frame* frame)
 {
     m_client->focusedFrameChanged(frame);
 }
 
-Page* Chrome::createWindow(LocalFrame& frame, const WindowFeatures& features, const NavigationAction& action)
+RefPtr<Page> Chrome::createWindow(LocalFrame& frame, const String& openedMainFrameName, const WindowFeatures& features, const NavigationAction& action)
 {
-    Page* newPage = m_client->createWindow(frame, features, action);
+    RefPtr newPage = m_client->createWindow(frame, openedMainFrameName, features, action);
     if (!newPage)
         return nullptr;
 
-    if (!features.noopener && !features.noreferrer)
-        m_page.storageNamespaceProvider().copySessionStorageNamespace(m_page, *newPage);
-
     return newPage;
 }
 
@@ -223,11 +241,11 @@ void Chrome::runModal()
     // JavaScript that runs within the nested event loop must not be run in the context of the
     // script that called showModalDialog. Null out entryScope to break the connection.
 
-    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame());
-    if (!localMainFrame)
+    RefPtr localTopDocument = m_page->localTopDocument();
+    if (!localTopDocument)
         return;
 
-    SetForScope entryScopeNullifier { localMainFrame->document()->vm().entryScope, nullptr };
+    SetForScope entryScopeNullifier { localTopDocument->vm().entryScope, nullptr };
 
     TimerBase::fireTimersInNestedEventLoop();
     m_client->runModal();
@@ -283,13 +301,13 @@ bool Chrome::canRunBeforeUnloadConfirmPanel()
     return m_client->canRunBeforeUnloadConfirmPanel();
 }
 
-bool Chrome::runBeforeUnloadConfirmPanel(const String& message, LocalFrame& frame)
+bool Chrome::runBeforeUnloadConfirmPanel(String&& message, LocalFrame& frame)
 {
     // Defer loads in case the client method runs a new event loop that would
     // otherwise cause the load to continue while we're in the middle of executing JavaScript.
     PageGroupLoadDeferrer deferrer(m_page, true);
 
-    return m_client->runBeforeUnloadConfirmPanel(message, frame);
+    return m_client->runBeforeUnloadConfirmPanel(WTFMove(message), frame);
 }
 
 void Chrome::closeWindow()
@@ -313,7 +331,7 @@ bool Chrome::runJavaScriptConfirm(LocalFrame& frame, const String& message)
 {
     // Defer loads in case the client method runs a new event loop that would
     // otherwise cause the load to continue while we're in the middle of executing JavaScript.
-    PageGroupLoadDeferrer deferrer(m_page, true);
+    PageGroupLoadDeferrer deferrer(protectedPage(), true);
 
     notifyPopupOpeningObservers();
     return m_client->runJavaScriptConfirm(frame, frame.displayStringModifiedByEncoding(message));
@@ -323,7 +341,7 @@ bool Chrome::runJavaScriptPrompt(LocalFrame& frame, const String& prompt, const
 {
     // Defer loads in case the client method runs a new event loop that would
     // otherwise cause the load to continue while we're in the middle of executing JavaScript.
-    PageGroupLoadDeferrer deferrer(m_page, true);
+    PageGroupLoadDeferrer deferrer(protectedPage(), true);
 
     notifyPopupOpeningObservers();
     String displayPrompt = frame.displayStringModifiedByEncoding(prompt);
@@ -335,26 +353,14 @@ bool Chrome::runJavaScriptPrompt(LocalFrame& frame, const String& prompt, const
     return ok;
 }
 
-void Chrome::setStatusbarText(LocalFrame& frame, const String& status)
-{
-    m_client->setStatusbarText(frame.displayStringModifiedByEncoding(status));
-}
-
 void Chrome::mouseDidMoveOverElement(const HitTestResult& result, OptionSet<PlatformEventModifier> modifiers)
 {
-    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame());
-    if (!localMainFrame)
-        return;
-
-    if (result.innerNode() && result.innerNode()->document().isDNSPrefetchEnabled())
-        localMainFrame->loader().client().prefetchDNS(result.absoluteLinkURL().host().toString());
-
     String toolTip;
     TextDirection toolTipDirection;
     getToolTip(result, toolTip, toolTipDirection);
     m_client->mouseDidMoveOverElement(result, modifiers, toolTip, toolTipDirection);
 
-    InspectorInstrumentation::mouseDidMoveOverElement(m_page, result, modifiers);
+    InspectorInstrumentation::mouseDidMoveOverElement(protectedPage(), result, modifiers);
 }
 
 void Chrome::getToolTip(const HitTestResult& result, String& toolTip, TextDirection& toolTipDirection)
@@ -363,16 +369,15 @@ void Chrome::getToolTip(const HitTestResult& result, String& toolTip, TextDirect
     toolTip = result.spellingToolTip(toolTipDirection);
 
     // Next priority is a toolTip from a URL beneath the mouse (if preference is set to show those).
-    if (toolTip.isEmpty() && m_page.settings().showsURLsInToolTips()) {
-        if (Element* element = result.innerNonSharedElement()) {
+    if (toolTip.isEmpty() && m_page->settings().showsURLsInToolTips()) {
+        if (RefPtr element = result.innerNonSharedElement()) {
             // Get tooltip representing form action, if relevant
-            if (is<HTMLInputElement>(*element)) {
-                HTMLInputElement& input = downcast<HTMLInputElement>(*element);
-                if (input.isSubmitButton()) {
-                    if (HTMLFormElement* form = input.form()) {
+            if (RefPtr input = dynamicDowncast<HTMLInputElement>(*element)) {
+                if (input->isSubmitButton()) {
+                    if (RefPtr form = input->form()) {
                         toolTip = form->action();
                         if (form->renderer())
-                            toolTipDirection = form->renderer()->style().direction();
+                            toolTipDirection = form->renderer()->writingMode().computedTextDirection();
                         else
                             toolTipDirection = TextDirection::LTR;
                     }
@@ -393,14 +398,14 @@ void Chrome::getToolTip(const HitTestResult& result, String& toolTip, TextDirect
     if (toolTip.isEmpty())
         toolTip = result.title(toolTipDirection);
 
-    if (toolTip.isEmpty() && m_page.settings().showsToolTipOverTruncatedText())
+    if (toolTip.isEmpty() && m_page->settings().showsToolTipOverTruncatedText())
         toolTip = result.innerTextIfTruncated(toolTipDirection);
 
     // Lastly, for <input type="file"> that allow multiple files, we'll consider a tooltip for the selected filenames
     if (toolTip.isEmpty()) {
-        if (Element* element = result.innerNonSharedElement()) {
-            if (is<HTMLInputElement>(*element)) {
-                toolTip = downcast<HTMLInputElement>(*element).defaultToolTip();
+        if (RefPtr element = result.innerNonSharedElement()) {
+            if (RefPtr input = dynamicDowncast<HTMLInputElement>(*element)) {
+                toolTip = input->defaultToolTip();
 
                 // FIXME: We should obtain text direction of tooltip from
                 // ChromeClient or platform. As of October 2011, all client
@@ -417,8 +422,8 @@ bool Chrome::print(LocalFrame& frame)
 {
     // FIXME: This should have PageGroupLoadDeferrer, like runModal() or runJavaScriptAlert(), because it's no different from those.
 
-    if (frame.document()->isSandboxed(SandboxModals)) {
-        frame.document()->domWindow()->printErrorMessage("Use of window.print is not allowed in a sandboxed frame when the allow-modals flag is not set."_s);
+    if (frame.document()->isSandboxed(SandboxFlag::Modals)) {
+        frame.document()->protectedWindow()->printErrorMessage("Use of window.print is not allowed in a sandboxed frame when the allow-modals flag is not set."_s);
         return false;
     }
 
@@ -436,9 +441,7 @@ void Chrome::disableSuddenTermination()
     m_client->disableSuddenTermination();
 }
 
-#if ENABLE(INPUT_TYPE_COLOR)
-
-std::unique_ptr<ColorChooser> Chrome::createColorChooser(ColorChooserClient& client, const Color& initialColor)
+RefPtr<ColorChooser> Chrome::createColorChooser(ColorChooserClient& client, const Color& initialColor)
 {
 #if PLATFORM(IOS_FAMILY)
     UNUSED_PARAM(client);
@@ -450,21 +453,13 @@ std::unique_ptr<ColorChooser> Chrome::createColorChooser(ColorChooserClient& cli
 #endif
 }
 
-#endif
-
-#if ENABLE(DATALIST_ELEMENT)
-
-std::unique_ptr<DataListSuggestionPicker> Chrome::createDataListSuggestionPicker(DataListSuggestionsClient& client)
+RefPtr<DataListSuggestionPicker> Chrome::createDataListSuggestionPicker(DataListSuggestionsClient& client)
 {
     notifyPopupOpeningObservers();
     return m_client->createDataListSuggestionPicker(client);
 }
 
-#endif
-
-#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
-
-std::unique_ptr<DateTimeChooser> Chrome::createDateTimeChooser(DateTimeChooserClient& client)
+RefPtr<DateTimeChooser> Chrome::createDateTimeChooser(DateTimeChooserClient& client)
 {
 #if PLATFORM(IOS_FAMILY)
     UNUSED_PARAM(client);
@@ -475,24 +470,34 @@ std::unique_ptr<DateTimeChooser> Chrome::createDateTimeChooser(DateTimeChooserCl
 #endif
 }
 
-#endif
-
 void Chrome::runOpenPanel(LocalFrame& frame, FileChooser& fileChooser)
 {
     notifyPopupOpeningObservers();
     m_client->runOpenPanel(frame, fileChooser);
 }
 
-void Chrome::showShareSheet(ShareDataWithParsedURL& shareData, CompletionHandler<void(bool)>&& callback)
+void Chrome::showShareSheet(ShareDataWithParsedURL&& shareData, CompletionHandler<void(bool)>&& callback)
 {
-    m_client->showShareSheet(shareData, WTFMove(callback));
+    m_client->showShareSheet(WTFMove(shareData), WTFMove(callback));
 }
 
-void Chrome::showContactPicker(const ContactsRequestData& requestData, CompletionHandler<void(std::optional<Vector<ContactInfo>>&&)>&& callback)
+void Chrome::showContactPicker(ContactsRequestData&& requestData, CompletionHandler<void(std::optional<Vector<ContactInfo>>&&)>&& callback)
 {
-    m_client->showContactPicker(requestData, WTFMove(callback));
+    m_client->showContactPicker(WTFMove(requestData), WTFMove(callback));
 }
 
+#if HAVE(DIGITAL_CREDENTIALS_UI)
+void Chrome::showDigitalCredentialsPicker(const DigitalCredentialsRequestData& requestData, WTF::CompletionHandler<void(Expected<WebCore::DigitalCredentialsResponseData, WebCore::ExceptionData>&&)>&& callback)
+{
+    m_client->showDigitalCredentialsPicker(requestData, WTFMove(callback));
+}
+
+void Chrome::dismissDigitalCredentialsPicker(CompletionHandler<void(bool)>&& callback)
+{
+    m_client->dismissDigitalCredentialsPicker(WTFMove(callback));
+}
+#endif
+
 void Chrome::loadIconForFiles(const Vector<String>& filenames, FileIconLoader& loader)
 {
     m_client->loadIconForFiles(filenames, loader);
@@ -513,6 +518,11 @@ FloatSize Chrome::overrideScreenSize() const
     return m_client->overrideScreenSize();
 }
 
+FloatSize Chrome::overrideAvailableScreenSize() const
+{
+    return m_client->overrideAvailableScreenSize();
+}
+
 void Chrome::dispatchDisabledAdaptationsDidChange(const OptionSet<DisabledAdaptations>& disabledAdaptations) const
 {
     m_client->dispatchDisabledAdaptationsDidChange(disabledAdaptations);
@@ -527,13 +537,6 @@ void Chrome::dispatchViewportPropertiesDidChange(const ViewportArguments& argume
     m_client->dispatchViewportPropertiesDidChange(arguments);
 }
 
-#if ENABLE(APP_HIGHLIGHTS)
-void Chrome::storeAppHighlight(AppHighlight&& highlight) const
-{
-    m_client->storeAppHighlight(WTFMove(highlight));
-}
-#endif
-
 void Chrome::setCursor(const Cursor& cursor)
 {
     m_client->setCursor(cursor);
@@ -544,9 +547,9 @@ void Chrome::setCursorHiddenUntilMouseMoves(bool hiddenUntilMouseMoves)
     m_client->setCursorHiddenUntilMouseMoves(hiddenUntilMouseMoves);
 }
 
-RefPtr<ImageBuffer> Chrome::createImageBuffer(const FloatSize& size, RenderingMode renderingMode, RenderingPurpose purpose, float resolutionScale, const DestinationColorSpace& colorSpace, PixelFormat pixelFormat, bool avoidBackendSizeCheck) const
+RefPtr<ImageBuffer> Chrome::createImageBuffer(const FloatSize& size, RenderingMode renderingMode, RenderingPurpose purpose, float resolutionScale, const DestinationColorSpace& colorSpace, ImageBufferFormat pixelFormat) const
 {
-    return m_client->createImageBuffer(size, renderingMode, purpose, resolutionScale, colorSpace, pixelFormat, avoidBackendSizeCheck);
+    return m_client->createImageBuffer(size, renderingMode, purpose, resolutionScale, colorSpace, pixelFormat);
 }
 
 RefPtr<ImageBuffer> Chrome::sinkIntoImageBuffer(std::unique_ptr<SerializedImageBuffer> imageBuffer)
@@ -565,11 +568,12 @@ RefPtr<GraphicsContextGL> Chrome::createGraphicsContextGL(const GraphicsContextG
     return m_client->createGraphicsContextGL(attributes);
 }
 #endif
-
+#if HAVE(WEBGPU_IMPLEMENTATION)
 RefPtr<WebGPU::GPU> Chrome::createGPUForWebGPU() const
 {
     return m_client->createGPUForWebGPU();
 }
+#endif
 
 RefPtr<ShapeDetection::BarcodeDetector> Chrome::createBarcodeDetector(const ShapeDetection::BarcodeDetectorOptions& barcodeDetectorOptions) const
 {
@@ -593,12 +597,12 @@ RefPtr<ShapeDetection::TextDetector> Chrome::createTextDetector() const
 
 PlatformDisplayID Chrome::displayID() const
 {
-    return m_page.displayID();
+    return m_page->displayID();
 }
 
 void Chrome::windowScreenDidChange(PlatformDisplayID displayID, std::optional<FramesPerSecond> nominalFrameInterval)
 {
-    m_page.windowScreenDidChange(displayID, nominalFrameInterval);
+    protectedPage()->windowScreenDidChange(displayID, nominalFrameInterval);
 }
 
 bool Chrome::selectItemWritingDirectionIsNatural()
@@ -643,18 +647,18 @@ void Chrome::didReceiveDocType(LocalFrame& frame)
 
 void Chrome::registerPopupOpeningObserver(PopupOpeningObserver& observer)
 {
-    m_popupOpeningObservers.append(&observer);
+    m_popupOpeningObservers.append(observer);
 }
 
 void Chrome::unregisterPopupOpeningObserver(PopupOpeningObserver& observer)
 {
-    bool removed = m_popupOpeningObservers.removeFirst(&observer);
+    bool removed = m_popupOpeningObservers.removeFirst(WeakPtr { observer });
     ASSERT_UNUSED(removed, removed);
 }
 
 void Chrome::notifyPopupOpeningObservers() const
 {
-    const Vector<PopupOpeningObserver*> observers(m_popupOpeningObservers);
+    auto observers = m_popupOpeningObservers;
     for (auto& observer : observers)
         observer->willOpenPopup();
 }
