@@ -20,28 +20,50 @@
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
  * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #pragma once
 
 #if ENABLE(MEDIA_SOURCE)
 
+#include "AudioTrackPrivate.h"
+#include "InbandTextTrackPrivate.h"
 #include "MediaDescription.h"
+#include "PlatformMediaError.h"
+#include "VideoTrackPrivate.h"
 #include <wtf/MediaTime.h>
+#include <wtf/Ref.h>
 #include <wtf/Vector.h>
 #include <wtf/WeakPtr.h>
+#include <wtf/text/MakeString.h>
 
 namespace WebCore {
 
-class AudioTrackPrivate;
-class InbandTextTrackPrivate;
 class MediaSample;
 class MediaDescription;
 class PlatformTimeRanges;
-class VideoTrackPrivate;
 
-class SourceBufferPrivateClient : public CanMakeWeakPtr<SourceBufferPrivateClient> {
+struct SourceBufferEvictionData {
+    uint64_t contentSize { 0 };
+    int64_t evictableSize { 0 };
+    uint64_t maximumBufferSize { 0 };
+    uint64_t numMediaSamples { 0 };
+
+    bool operator!=(const SourceBufferEvictionData& other)
+    {
+        return contentSize != other.contentSize || evictableSize != other.evictableSize || maximumBufferSize != other.maximumBufferSize || numMediaSamples != other.numMediaSamples;
+    }
+
+    void clear()
+    {
+        contentSize = 0;
+        evictableSize = 0;
+        numMediaSamples = 0;
+    }
+};
+
+class SourceBufferPrivateClient : public ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<SourceBufferPrivateClient> {
 public:
     virtual ~SourceBufferPrivateClient() = default;
 
@@ -51,84 +73,52 @@ public:
         struct AudioTrackInformation {
             RefPtr<MediaDescription> description;
             RefPtr<AudioTrackPrivate> track;
+
+            RefPtr<MediaDescription> protectedDescription() const { return description; }
+            RefPtr<AudioTrackPrivate> protectedTrack() const { return track; }
         };
         Vector<AudioTrackInformation> audioTracks;
 
         struct VideoTrackInformation {
             RefPtr<MediaDescription> description;
             RefPtr<VideoTrackPrivate> track;
+
+            RefPtr<MediaDescription> protectedDescription() const { return description; }
+            RefPtr<VideoTrackPrivate> protectedTrack() const { return track; }
         };
         Vector<VideoTrackInformation> videoTracks;
 
         struct TextTrackInformation {
             RefPtr<MediaDescription> description;
             RefPtr<InbandTextTrackPrivate> track;
+
+            RefPtr<MediaDescription> protectedDescription() const { return description; }
+            RefPtr<InbandTextTrackPrivate> protectedTrack() const { return track; }
         };
         Vector<TextTrackInformation> textTracks;
     };
-    
-    enum class ReceiveResult : uint8_t {
-        RecieveSucceeded,
-        AppendError,
-        ClientDisconnected,
-        BufferRemoved,
-        IPCError,
-    };
-    virtual void sourceBufferPrivateDidReceiveInitializationSegment(InitializationSegment&&, CompletionHandler<void(ReceiveResult)>&&) = 0;
-    virtual void sourceBufferPrivateStreamEndedWithDecodeError() = 0;
-    virtual void sourceBufferPrivateAppendError(bool decodeError) = 0;
-    enum class AppendResult : uint8_t {
-        AppendSucceeded,
-        ReadStreamFailed,
-        ParsingFailed
-    };
-    virtual void sourceBufferPrivateAppendComplete(AppendResult) = 0;
-    virtual void sourceBufferPrivateDurationChanged(const MediaTime&, CompletionHandler<void()>&&) = 0;
-    virtual void sourceBufferPrivateHighestPresentationTimestampChanged(const MediaTime&) = 0;
-    virtual void sourceBufferPrivateDidParseSample(double frameDuration) = 0;
-    virtual void sourceBufferPrivateDidDropSample() = 0;
-    virtual void sourceBufferPrivateBufferedDirtyChanged(bool) = 0;
-    virtual void sourceBufferPrivateDidReceiveRenderingError(int64_t errorCode) = 0;
-    virtual void sourceBufferPrivateReportExtraMemoryCost(uint64_t) = 0;
-};
 
-String convertEnumerationToString(SourceBufferPrivateClient::ReceiveResult);
+    virtual Ref<MediaPromise> sourceBufferPrivateDidReceiveInitializationSegment(InitializationSegment&&) = 0;
+    virtual Ref<MediaPromise> sourceBufferPrivateBufferedChanged(const Vector<PlatformTimeRanges>&) = 0;
+    virtual Ref<MediaPromise> sourceBufferPrivateDurationChanged(const MediaTime&) = 0;
+    virtual void sourceBufferPrivateHighestPresentationTimestampChanged(const MediaTime&) = 0;
+    virtual void sourceBufferPrivateDidDropSample() = 0;
+    virtual void sourceBufferPrivateDidReceiveRenderingError(int64_t errorCode) = 0;
+    virtual void sourceBufferPrivateEvictionDataChanged(const SourceBufferEvictionData&) { }
+    virtual Ref<MediaPromise> sourceBufferPrivateDidAttach(InitializationSegment&&) = 0;
+};
 
 } // namespace WebCore
 
 namespace WTF {
-
-template<typename Type>
-struct LogArgument;
-
-template <>
-struct LogArgument<WebCore::SourceBufferPrivateClient::ReceiveResult> {
-    static String toString(const WebCore::SourceBufferPrivateClient::ReceiveResult result)
+template<>
+struct LogArgument<WebCore::SourceBufferEvictionData> {
+    static String toString(const WebCore::SourceBufferEvictionData& evictionData)
     {
-        return convertEnumerationToString(result);
+        return makeString("{ contentSize:"_s, evictionData.contentSize, " evictableData:"_s, evictionData.evictableSize, " maximumBufferSize:"_s, evictionData.maximumBufferSize, " numSamples:"_s, evictionData.numMediaSamples, " }"_s);
     }
 };
 
-template<> struct EnumTraits<WebCore::SourceBufferPrivateClient::ReceiveResult> {
-    using values = EnumValues<
-        WebCore::SourceBufferPrivateClient::ReceiveResult,
-        WebCore::SourceBufferPrivateClient::ReceiveResult::RecieveSucceeded,
-        WebCore::SourceBufferPrivateClient::ReceiveResult::AppendError,
-        WebCore::SourceBufferPrivateClient::ReceiveResult::ClientDisconnected,
-        WebCore::SourceBufferPrivateClient::ReceiveResult::BufferRemoved,
-        WebCore::SourceBufferPrivateClient::ReceiveResult::IPCError
-    >;
-};
-
-template<> struct EnumTraits<WebCore::SourceBufferPrivateClient::AppendResult> {
-    using values = EnumValues<
-        WebCore::SourceBufferPrivateClient::AppendResult,
-        WebCore::SourceBufferPrivateClient::AppendResult::AppendSucceeded,
-        WebCore::SourceBufferPrivateClient::AppendResult::ReadStreamFailed,
-        WebCore::SourceBufferPrivateClient::AppendResult::ParsingFailed
-    >;
-};
-
 } // namespace WTF
 
 #endif // ENABLE(MEDIA_SOURCE)
