@@ -45,57 +45,57 @@ TEST(WTF_StringCommon, Find8NonASCII)
     EXPECT_FALSE(WTF::find8NonASCII(vector.subspan(0, 4096)));
 
     vector[4095] = 0x80;
-    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.data(), 4095);
+    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 4095);
     for (unsigned i = 0; i < 16; ++i)
         EXPECT_FALSE(WTF::find8NonASCII(vector.subspan(0, 4095 - i)));
 
     vector[1024] = 0x80;
-    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.data(), 1024);
+    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 1024);
     EXPECT_FALSE(WTF::find8NonASCII(vector.subspan(0, 1023)));
 
     vector[1024] = 0xff;
-    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.data(), 1024);
+    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 1024);
     EXPECT_FALSE(WTF::find8NonASCII(vector.subspan(0, 1023)));
 
     vector[1024] = 0x7f;
-    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.data(), 4095);
+    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 4095);
 
     vector[0] = 0xff;
-    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.data(), 0);
+    EXPECT_EQ(WTF::find8NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 0);
     for (int i = 0; i < 16; ++i) {
         vector[i] = 0xff;
-        EXPECT_EQ(WTF::find8NonASCII(vector.subspan(i, 4096 - i)) - vector.data(), i);
+        EXPECT_EQ(WTF::find8NonASCII(vector.subspan(i, 4096 - i)) - vector.span().data(), i);
     }
 }
 
 TEST(WTF_StringCommon, Find16NonASCII)
 {
-    Vector<UChar> vector(4096);
+    Vector<char16_t> vector(4096);
     vector.fill('a');
 
     EXPECT_FALSE(WTF::find16NonASCII(vector.subspan(0, 4096)));
 
     vector[4095] = 0x80;
-    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.data(), 4095);
+    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 4095);
     for (unsigned i = 0; i < 16; ++i)
         EXPECT_FALSE(WTF::find16NonASCII(vector.subspan(0, 4095 - i)));
 
     vector[1024] = 0x80;
-    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.data(), 1024);
+    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 1024);
     EXPECT_FALSE(WTF::find16NonASCII(vector.subspan(0, 1023)));
 
     vector[1024] = 0xff;
-    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.data(), 1024);
+    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 1024);
     EXPECT_FALSE(WTF::find16NonASCII(vector.subspan(0, 1023)));
 
     vector[1024] = 0x7f;
-    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.data(), 4095);
+    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 4095);
 
     vector[0] = 0xff;
-    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.data(), 0);
+    EXPECT_EQ(WTF::find16NonASCII(vector.subspan(0, 4096)) - vector.span().data(), 0);
     for (int i = 0; i < 16; ++i) {
         vector[i] = 0xff;
-        EXPECT_EQ(WTF::find16NonASCII(vector.subspan(i, 4096 - i)) - vector.data(), i);
+        EXPECT_EQ(WTF::find16NonASCII(vector.subspan(i, 4096 - i)) - vector.span().data(), i);
     }
 }
 #endif
@@ -247,67 +247,67 @@ TEST(WTF_StringCommon, CharactersContain8)
 TEST(WTF_StringCommon, CharactersContain16)
 {
     {
-        Vector<UChar> source;
-        EXPECT_FALSE((charactersContain<UChar, 0>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0, 1>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0, 1, 2>(source.span())));
+        Vector<char16_t> source;
+        EXPECT_FALSE((charactersContain<char16_t, 0>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0, 1>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0, 1, 2>(source.span())));
     }
 
     {
-        Vector<UChar> source;
+        Vector<char16_t> source;
         for (unsigned i = 0; i < 15; ++i)
             source.append(i);
-        EXPECT_TRUE((charactersContain<UChar, 0>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 1>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 2>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 2, 3>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 16, 14>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 16>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 16, 15>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 16, 15, 17>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 16, 15, 17, 18>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x81>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x81, 0x82>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 0>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 1>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 2>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 2, 3>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 16, 14>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 16>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 16, 15>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 16, 15, 17>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 16, 15, 17, 18>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x81>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x81, 0x82>(source.span())));
     }
 
     {
-        Vector<UChar> source;
+        Vector<char16_t> source;
         for (unsigned i = 0; i < 250; ++i) {
             if (i & 0x1)
                 source.append(i);
         }
-        EXPECT_FALSE((charactersContain<UChar, 0>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0xff>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 0x81>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 250>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 249>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 0, 249>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x101>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x1001>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x1001, 0x1001>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0xff>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 0x81>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 250>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 249>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 0, 249>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x101>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x1001>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x1001, 0x1001>(source.span())));
     }
 
     {
-        Vector<UChar> source;
+        Vector<char16_t> source;
         for (unsigned i = 0; i < 250; ++i) {
             if (i & 0x1)
                 source.append(i + 0x1000);
         }
-        EXPECT_FALSE((charactersContain<UChar, 0>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0xff>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x81>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 250>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 249>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x101>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 0x1001>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x1000>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x1100>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x1000 + 256>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x1000 + 250>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 0x1000 + 249>(source.span())));
-        EXPECT_TRUE((charactersContain<UChar, 0x1000 + 249, 0>(source.span())));
-        EXPECT_FALSE((charactersContain<UChar, 0x1000 + 250, 0>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0xff>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x81>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 250>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 249>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x101>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 0x1001>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x1000>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x1100>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x1000 + 256>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x1000 + 250>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 0x1000 + 249>(source.span())));
+        EXPECT_TRUE((charactersContain<char16_t, 0x1000 + 249, 0>(source.span())));
+        EXPECT_FALSE((charactersContain<char16_t, 0x1000 + 250, 0>(source.span())));
     }
 }
 
@@ -386,61 +386,61 @@ TEST(WTF_StringCommon, CountMatchedCharacters8)
 TEST(WTF_StringCommon, CountMatchedCharacters16)
 {
     {
-        Vector<UChar> source;
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 1)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 2)), 0U);
+        Vector<char16_t> source;
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 1)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 2)), 0U);
     }
 
     {
-        Vector<UChar> source;
+        Vector<char16_t> source;
         for (unsigned i = 0; i < 15; ++i)
             source.append(i);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0)), 1U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 1)), 1U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 2)), 1U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 3)), 1U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 14)), 1U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 15)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 16)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 17)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 18)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0x81)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0x82)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 1)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 2)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 3)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 14)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 15)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 16)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 17)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 18)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0x81)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0x82)), 0U);
     }
 
     {
-        Vector<UChar> source;
+        Vector<char16_t> source;
         for (unsigned i = 0; i < 250; ++i) {
             if (i & 0x1)
                 source.append(i);
         }
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 1)), 1U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0xff)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0x81)), 1U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 250)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 249)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 1)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0xff)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0x81)), 1U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 250)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 249)), 1U);
     }
 
     {
-        Vector<UChar> source;
+        Vector<char16_t> source;
         for (unsigned c = 0; c < 1024; ++c) {
             for (unsigned i = 0; i < 250; ++i) {
                 if (i & 0x1)
                     source.append(i);
             }
         }
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 1)), 1024U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0xff)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0x81)), 1024U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 250)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 249)), 1024U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 1)), 1024U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0xff)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0x81)), 1024U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 250)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 249)), 1024U);
     }
 
     {
-        Vector<UChar> source;
+        Vector<char16_t> source;
         for (unsigned c = 0; c < 0xffff; ++c) {
             for (unsigned i = 0; i < 250; ++i)
                 source.append(1);
@@ -449,9 +449,9 @@ TEST(WTF_StringCommon, CountMatchedCharacters16)
         source.append(1);
         source.append(1);
 
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0)), 0U);
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 1)), source.size());
-        EXPECT_EQ((WTF::countMatchedCharacters<UChar>(source.span(), 0x81)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0)), 0U);
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 1)), source.size());
+        EXPECT_EQ((WTF::countMatchedCharacters<char16_t>(source.span(), 0x81)), 0U);
     }
 }
 
