@@ -52,12 +52,11 @@ public:
     {
         return adoptRef(*new TestSample(presentationTime, decodeTime, duration, flags));
     }
-    
+
     MediaTime presentationTime() const final { return m_presentationTime; }
     MediaTime decodeTime() const final { return m_decodeTime; }
     MediaTime duration() const final { return m_duration; }
-    AtomString trackID() const final { return m_trackID; }
-    void setTrackID(const String& trackID) final { m_trackID = trackID; }
+    TrackID trackID() const final { return m_trackID; }
     size_t sizeInBytes() const final { return m_sizeInBytes; }
     FloatSize presentationSize() const final { return m_presentationSize; }
     void offsetTimestampsBy(const MediaTime& offset) final { m_presentationTime += offset; m_decodeTime += offset; }
@@ -65,13 +64,13 @@ public:
         m_presentationTime = presentationTime;
         m_decodeTime = decodeTime;
     };
-    bool isDivisable() const final { return false; }
-    std::pair<RefPtr<MediaSample>, RefPtr<MediaSample>> divide(const MediaTime& presentationTime) final { return { }; }
     Ref<MediaSample> createNonDisplayingCopy() const final {
         return create(m_presentationTime, m_decodeTime, m_duration, static_cast<SampleFlags>(m_flags | IsNonDisplaying));
     }
     SampleFlags flags() const final { return m_flags; }
-    PlatformSample platformSample() final { return { PlatformSample::None, {nullptr}}; }
+    PlatformSample platformSample() const final { return { PlatformSample::None, { nullptr } }; }
+    PlatformSample::Type platformSampleType() const final { return PlatformSample::None; }
+
     void dump(PrintStream&) const final { }
 
 private:
@@ -87,7 +86,7 @@ private:
     MediaTime m_decodeTime;
     MediaTime m_duration;
     FloatSize m_presentationSize;
-    AtomString m_trackID;
+    TrackID m_trackID;
     size_t m_sizeInBytes { 0 };
     SampleFlags m_flags { None };
 };
@@ -96,25 +95,35 @@ class SampleMapTest : public testing::Test {
 public:
     void SetUp() final {
         map.addSample(TestSample::create(MediaTime(0, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::IsSync));
-        map.addSample(TestSample::create(MediaTime(1, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(2, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(3, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(4, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(5, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::IsSync));
-        map.addSample(TestSample::create(MediaTime(6, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(7, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(8, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(9, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(1, 1), MediaTime(1, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(2, 1), MediaTime(2, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(3, 1), MediaTime(3, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(4, 1), MediaTime(4, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(5, 1), MediaTime(5, 1), MediaTime(1, 1), MediaSample::IsSync));
+        map.addSample(TestSample::create(MediaTime(6, 1), MediaTime(6, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(7, 1), MediaTime(7, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(8, 1), MediaTime(8, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(9, 1), MediaTime(9, 1), MediaTime(1, 1), MediaSample::None));
         // Gap at MediaTime(10, 1) -> MediaTime(11, 1);
-        map.addSample(TestSample::create(MediaTime(11, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::IsSync));
-        map.addSample(TestSample::create(MediaTime(12, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(13, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(14, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(15, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::IsSync));
-        map.addSample(TestSample::create(MediaTime(16, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(17, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(18, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
-        map.addSample(TestSample::create(MediaTime(19, 1), MediaTime(0, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(11, 1), MediaTime(11, 1), MediaTime(1, 1), MediaSample::IsSync));
+        map.addSample(TestSample::create(MediaTime(12, 1), MediaTime(12, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(13, 1), MediaTime(13, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(14, 1), MediaTime(14, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(15, 1), MediaTime(15, 1), MediaTime(1, 1), MediaSample::IsSync));
+        map.addSample(TestSample::create(MediaTime(16, 1), MediaTime(16, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(17, 1), MediaTime(17, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(18, 1), MediaTime(18, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(19, 1), MediaTime(19, 1), MediaTime(1, 1), MediaSample::None));
+        // Some B-frames
+        map.addSample(TestSample::create(MediaTime(20, 1), MediaTime(20, 1), MediaTime(1, 1), MediaSample::IsSync));
+        map.addSample(TestSample::create(MediaTime(23, 1), MediaTime(21, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(22, 1), MediaTime(22, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(21, 1), MediaTime(23, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(24, 1), MediaTime(24, 1), MediaTime(1, 1), MediaSample::IsSync));
+        map.addSample(TestSample::create(MediaTime(28, 1), MediaTime(25, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(27, 1), MediaTime(26, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(26, 1), MediaTime(27, 1), MediaTime(1, 1), MediaSample::None));
+        map.addSample(TestSample::create(MediaTime(25, 1), MediaTime(28, 1), MediaTime(1, 1), MediaSample::None));
     }
 
     SampleMap map;
@@ -125,9 +134,10 @@ TEST_F(SampleMapTest, findSampleWithPresentationTime)
     auto& presentationMap = map.presentationOrder();
     EXPECT_EQ(MediaTime(0, 1), presentationMap.findSampleWithPresentationTime(MediaTime(0, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(19, 1), presentationMap.findSampleWithPresentationTime(MediaTime(19, 1))->second->presentationTime());
+    EXPECT_EQ(MediaTime(26, 1), presentationMap.findSampleWithPresentationTime(MediaTime(26, 1))->second->presentationTime());
     EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleWithPresentationTime(MediaTime(-1, 1)));
     EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleWithPresentationTime(MediaTime(10, 1)));
-    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleWithPresentationTime(MediaTime(20, 1)));
+    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleWithPresentationTime(MediaTime(29, 1)));
     EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleWithPresentationTime(MediaTime(1, 2)));
 }
 
@@ -137,9 +147,10 @@ TEST_F(SampleMapTest, findSampleContainingPresentationTime)
     EXPECT_EQ(MediaTime(0, 1), presentationMap.findSampleContainingPresentationTime(MediaTime(0, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(19, 1), presentationMap.findSampleContainingPresentationTime(MediaTime(19, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(0, 1), presentationMap.findSampleContainingPresentationTime(MediaTime(1, 2))->second->presentationTime());
+    EXPECT_EQ(MediaTime(26, 1), presentationMap.findSampleContainingPresentationTime(MediaTime(26, 1) + MediaTime(1, 2))->second->presentationTime());
     EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingPresentationTime(MediaTime(-1, 1)));
-    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingPresentationTime(MediaTime(10, 1)));
-    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingPresentationTime(MediaTime(20, 1)));
+    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingPresentationTime(MediaTime(29, 1)));
+    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingPresentationTime(MediaTime(30, 1)));
 }
 
 TEST_F(SampleMapTest, findSampleStartingOnOrAfterPresentationTime)
@@ -150,7 +161,9 @@ TEST_F(SampleMapTest, findSampleStartingOnOrAfterPresentationTime)
     EXPECT_EQ(MediaTime(1, 1), presentationMap.findSampleStartingOnOrAfterPresentationTime(MediaTime(1, 2))->second->presentationTime());
     EXPECT_EQ(MediaTime(0, 1), presentationMap.findSampleStartingOnOrAfterPresentationTime(MediaTime(-1, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(11, 1), presentationMap.findSampleStartingOnOrAfterPresentationTime(MediaTime(10, 1))->second->presentationTime());
-    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingPresentationTime(MediaTime(20, 1)));
+    EXPECT_EQ(MediaTime(25, 1), presentationMap.findSampleStartingOnOrAfterPresentationTime(MediaTime(25, 1))->second->presentationTime());
+    EXPECT_EQ(MediaTime(26, 1), presentationMap.findSampleStartingOnOrAfterPresentationTime(MediaTime(25, 1) + MediaTime(1, 2))->second->presentationTime());
+    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleStartingOnOrAfterPresentationTime(MediaTime(30, 1)));
 }
 
 TEST_F(SampleMapTest, findSampleContainingOrAfterPresentationTime)
@@ -161,19 +174,23 @@ TEST_F(SampleMapTest, findSampleContainingOrAfterPresentationTime)
     EXPECT_EQ(MediaTime(0, 1), presentationMap.findSampleContainingOrAfterPresentationTime(MediaTime(1, 2))->second->presentationTime());
     EXPECT_EQ(MediaTime(0, 1), presentationMap.findSampleContainingOrAfterPresentationTime(MediaTime(-1, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(11, 1), presentationMap.findSampleContainingOrAfterPresentationTime(MediaTime(10, 1))->second->presentationTime());
-    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingOrAfterPresentationTime(MediaTime(20, 1)));
+    EXPECT_EQ(MediaTime(26, 1), presentationMap.findSampleContainingOrAfterPresentationTime(MediaTime(26, 1))->second->presentationTime());
+    EXPECT_EQ(MediaTime(25, 1), presentationMap.findSampleContainingOrAfterPresentationTime(MediaTime(25, 1) + MediaTime(1, 2))->second->presentationTime());
+    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleContainingOrAfterPresentationTime(MediaTime(30, 1)));
 }
 
 TEST_F(SampleMapTest, findSampleStartingAfterPresentationTime)
 {
     auto& presentationMap = map.presentationOrder();
     EXPECT_EQ(MediaTime(1, 1), presentationMap.findSampleStartingAfterPresentationTime(MediaTime(0, 1))->second->presentationTime());
-    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleStartingAfterPresentationTime(MediaTime(19, 1)));
     EXPECT_EQ(MediaTime(1, 1), presentationMap.findSampleStartingAfterPresentationTime(MediaTime(1, 2))->second->presentationTime());
     EXPECT_EQ(MediaTime(0, 1), presentationMap.findSampleStartingAfterPresentationTime(MediaTime(-1, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(11, 1), presentationMap.findSampleStartingAfterPresentationTime(MediaTime(10, 1))->second->presentationTime());
-    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleStartingAfterPresentationTime(MediaTime(20, 1)));
+    EXPECT_EQ(MediaTime(23, 1), presentationMap.findSampleStartingAfterPresentationTime(MediaTime(22, 1))->second->presentationTime());
+    EXPECT_EQ(MediaTime(26, 1), presentationMap.findSampleStartingAfterPresentationTime(MediaTime(25, 1))->second->presentationTime());
+    EXPECT_TRUE(presentationMap.end() == presentationMap.findSampleStartingAfterPresentationTime(MediaTime(30, 1)));
 }
+
 TEST_F(SampleMapTest, findSamplesBetweenPresentationTimes)
 {
     auto& presentationMap = map.presentationOrder();
@@ -185,7 +202,16 @@ TEST_F(SampleMapTest, findSamplesBetweenPresentationTimes)
     EXPECT_EQ(MediaTime(1, 1), iterator_range.first->second->presentationTime());
     EXPECT_EQ(MediaTime(2, 1), iterator_range.second->second->presentationTime());
 
-    iterator_range = presentationMap.findSamplesBetweenPresentationTimes(MediaTime(9, 1), MediaTime(21, 1));
+    iterator_range = presentationMap.findSamplesBetweenPresentationTimes(MediaTime(19, 1), MediaTime(25, 1));
+    EXPECT_EQ(MediaTime(19, 1), iterator_range.first->second->presentationTime());
+    EXPECT_EQ(MediaTime(25, 1), iterator_range.second->second->presentationTime());
+    MediaTime currentTime = iterator_range.first->second->presentationTime();
+    for (auto it = iterator_range.first; it != iterator_range.second; ++it) {
+        EXPECT_EQ(it->second->presentationTime(), currentTime);
+        currentTime += MediaTime { 1, 1 };
+    }
+
+    iterator_range = presentationMap.findSamplesBetweenPresentationTimes(MediaTime(9, 1), MediaTime(31, 1));
     EXPECT_EQ(MediaTime(9, 1), iterator_range.first->second->presentationTime());
     EXPECT_TRUE(presentationMap.end() == iterator_range.second);
 
@@ -197,7 +223,7 @@ TEST_F(SampleMapTest, findSamplesBetweenPresentationTimes)
     EXPECT_TRUE(presentationMap.end() == iterator_range.first);
     EXPECT_TRUE(presentationMap.end() == iterator_range.second);
 
-    iterator_range = presentationMap.findSamplesBetweenPresentationTimes(MediaTime(20, 1), MediaTime(21, 1));
+    iterator_range = presentationMap.findSamplesBetweenPresentationTimes(MediaTime(30, 1), MediaTime(31, 1));
     EXPECT_TRUE(presentationMap.end() == iterator_range.first);
     EXPECT_TRUE(presentationMap.end() == iterator_range.second);
 }
@@ -213,7 +239,16 @@ TEST_F(SampleMapTest, findSamplesBetweenPresentationTimesFromEnd)
     EXPECT_EQ(MediaTime(1, 1), iterator_range.first->second->presentationTime());
     EXPECT_EQ(MediaTime(2, 1), iterator_range.second->second->presentationTime());
 
-    iterator_range = presentationMap.findSamplesBetweenPresentationTimesFromEnd(MediaTime(9, 1), MediaTime(21, 1));
+    iterator_range = presentationMap.findSamplesBetweenPresentationTimesFromEnd(MediaTime(19, 1), MediaTime(25, 1));
+    EXPECT_EQ(MediaTime(19, 1), iterator_range.first->second->presentationTime());
+    EXPECT_EQ(MediaTime(25, 1), iterator_range.second->second->presentationTime());
+    MediaTime currentTime = iterator_range.first->second->presentationTime();
+    for (auto it = iterator_range.first; it != iterator_range.second; ++it) {
+        EXPECT_EQ(it->second->presentationTime(), currentTime);
+        currentTime += MediaTime { 1, 1 };
+    }
+
+    iterator_range = presentationMap.findSamplesBetweenPresentationTimesFromEnd(MediaTime(9, 1), MediaTime(31, 1));
     EXPECT_EQ(MediaTime(9, 1), iterator_range.first->second->presentationTime());
     EXPECT_TRUE(presentationMap.end() == iterator_range.second);
 
@@ -225,7 +260,7 @@ TEST_F(SampleMapTest, findSamplesBetweenPresentationTimesFromEnd)
     EXPECT_TRUE(presentationMap.end() == iterator_range.first);
     EXPECT_TRUE(presentationMap.end() == iterator_range.second);
 
-    iterator_range = presentationMap.findSamplesBetweenPresentationTimesFromEnd(MediaTime(20, 1), MediaTime(21, 1));
+    iterator_range = presentationMap.findSamplesBetweenPresentationTimesFromEnd(MediaTime(30, 1), MediaTime(31, 1));
     EXPECT_TRUE(presentationMap.end() == iterator_range.first);
     EXPECT_TRUE(presentationMap.end() == iterator_range.second);
 }
@@ -236,10 +271,79 @@ TEST_F(SampleMapTest, reverseFindSampleBeforePresentationTime)
     EXPECT_EQ(MediaTime(0, 1), presentationMap.reverseFindSampleBeforePresentationTime(MediaTime(0, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(9, 1), presentationMap.reverseFindSampleBeforePresentationTime(MediaTime(10, 1))->second->presentationTime());
     EXPECT_EQ(MediaTime(19, 1), presentationMap.reverseFindSampleBeforePresentationTime(MediaTime(19, 1))->second->presentationTime());
-    EXPECT_EQ(MediaTime(19, 1), presentationMap.reverseFindSampleBeforePresentationTime(MediaTime(21, 1))->second->presentationTime());
+    EXPECT_EQ(MediaTime(28, 1), presentationMap.reverseFindSampleBeforePresentationTime(MediaTime(31, 1))->second->presentationTime());
     EXPECT_TRUE(presentationMap.rend() == presentationMap.reverseFindSampleBeforePresentationTime(MediaTime(-1, 1)));
 }
 
+TEST_F(SampleMapTest, findSamplesBetweenDecodeKeys)
+{
+    auto& decodeMap = map.decodeOrder();
+    DecodeOrderSampleMap::MapType dependentSamples;
+    DecodeOrderSampleMap::KeyType decodeKeyStart(MediaTime(0, 1), MediaTime(0, 1));
+    DecodeOrderSampleMap::KeyType decodeKeyEnd(MediaTime(28, 1), MediaTime(25, 1));
+
+    auto samplesWithHigherDecodeTimes = decodeMap.findSamplesBetweenDecodeKeys(decodeKeyStart, decodeKeyEnd);
+    EXPECT_FALSE(samplesWithHigherDecodeTimes.isEmpty());
+    // Check that all samples returned are actually in the map.
+    for (auto& keyPair : samplesWithHigherDecodeTimes) {
+        auto iterator = decodeMap.findSampleWithDecodeKey(keyPair.first);
+        EXPECT_NE(iterator, decodeMap.end());
+    }
+
+    EXPECT_EQ(MediaTime(0, 1), samplesWithHigherDecodeTimes.begin()->second->presentationTime());
+    EXPECT_EQ(MediaTime(26, 1), samplesWithHigherDecodeTimes.rbegin()->second->presentationTime());
+    auto lastKey = samplesWithHigherDecodeTimes.rbegin()->first;
+    auto iterator = decodeMap.findSampleWithDecodeKey(lastKey);
+    EXPECT_NE(iterator, decodeMap.end());
+    EXPECT_NE(++iterator, decodeMap.end());
+    EXPECT_EQ(MediaTime(25, 1), iterator->second->presentationTime());
+    EXPECT_EQ(MediaTime(28, 1), iterator->second->decodeTime());
+
+    decodeKeyEnd = { MediaTime(25, 1), MediaTime(28, 1) };
+    samplesWithHigherDecodeTimes = decodeMap.findSamplesBetweenDecodeKeys(decodeKeyStart, decodeKeyEnd);
+    EXPECT_FALSE(samplesWithHigherDecodeTimes.isEmpty());
+    // Check that all samples returned are actually in the map.
+    for (auto& keyPair : samplesWithHigherDecodeTimes) {
+        auto iterator = decodeMap.findSampleWithDecodeKey(keyPair.first);
+        EXPECT_NE(iterator, decodeMap.end());
+    }
+    lastKey = samplesWithHigherDecodeTimes.rbegin()->first;
+    iterator = decodeMap.findSampleWithDecodeKey(lastKey);
+    EXPECT_NE(iterator, decodeMap.end());
+    EXPECT_NE(++iterator, decodeMap.end());
+    EXPECT_EQ(MediaTime(28, 1), iterator->second->presentationTime());
+
+    // Correctly not include key with the same decode time but lesser presentation time.
+    decodeKeyEnd = { MediaTime(28, 1), MediaTime(24, 1) };
+    samplesWithHigherDecodeTimes = decodeMap.findSamplesBetweenDecodeKeys(decodeKeyStart, decodeKeyEnd);
+    // Check that all samples returned are actually in the map.
+    for (auto& keyPair : samplesWithHigherDecodeTimes) {
+        auto iterator = decodeMap.findSampleWithDecodeKey(keyPair.first);
+        EXPECT_NE(iterator, decodeMap.end());
+    }
+    lastKey = samplesWithHigherDecodeTimes.rbegin()->first;
+    iterator = decodeMap.findSampleWithDecodeKey(lastKey);
+    EXPECT_NE(iterator, decodeMap.end());
+    EXPECT_NE(++iterator, decodeMap.end());
+    EXPECT_EQ(MediaTime(25, 1), iterator->second->presentationTime());
+    EXPECT_EQ(MediaTime(28, 1), iterator->second->decodeTime());
+
+    // Correctly includes the last element if decodeKeyEnd was after the end of the map.
+    decodeKeyEnd = { MediaTime(28, 1), MediaTime(28, 1) };
+    samplesWithHigherDecodeTimes = decodeMap.findSamplesBetweenDecodeKeys(decodeKeyStart, decodeKeyEnd);
+    // Check that all samples returned are actually in the map.
+    for (auto& keyPair : samplesWithHigherDecodeTimes) {
+        auto iterator = decodeMap.findSampleWithDecodeKey(keyPair.first);
+        EXPECT_NE(iterator, decodeMap.end());
+    }
+    EXPECT_FALSE(samplesWithHigherDecodeTimes.isEmpty());
+    EXPECT_EQ(*decodeMap.rbegin(), *samplesWithHigherDecodeTimes.rbegin());
+
+    decodeKeyEnd = { MediaTime(29, 1), MediaTime(30, 1) };
+    samplesWithHigherDecodeTimes = decodeMap.findSamplesBetweenDecodeKeys(decodeKeyStart, decodeKeyEnd);
+    EXPECT_FALSE(samplesWithHigherDecodeTimes.isEmpty());
+    EXPECT_EQ(*decodeMap.rbegin(), *samplesWithHigherDecodeTimes.rbegin());
 }
 
+}
 #endif // ENABLE(MEDIA_SOURCE)
